@@ -5,6 +5,7 @@
 #include <algorithm>
 #include <zypp/PoolQuery.h>
 #include <zypp/ResObjects.h>
+#include <zypp/ui/SelectableTraits.h>
 
 static std::string appname( "NameReqPrv" );
 
@@ -28,7 +29,7 @@ int usage( const std::string & msg_r = std::string(), int exit_r = 100 )
   }
   cerr << "Usage: " << appname << " [--root ROOTDIR] [OPTIONS] NAME... [[OPTIONS] NAME...]..." << endl;
   cerr << "  Load all enabled repositories (no refresh) and search for" << endl;
-  cerr << "  occurrences of NAME (regex) in package names or dependencies" << endl;
+  cerr << "  occurrences of NAME (regex or -x) in package names or dependencies" << endl;
   cerr << "  --root   Load repos from the system located below ROOTDIR. If ROOTDIR" << endl;
   cerr << "           denotes a sover testcase, the testcase is loaded." << endl;
   cerr << "  --installed Process installed packages only." << endl;
@@ -43,6 +44,7 @@ int usage( const std::string & msg_r = std::string(), int exit_r = 100 )
   cerr << "  -e/-E    turn on/off looking for enhan./sugg.(default off)" << endl;
   cerr << "  -a       short for -n -p -r" << endl;
   cerr << "  -A       short for -n -P -R" << endl;
+  cerr << "  -x       do exact matching (glob) rather than regex (substring)" << endl;
   cerr << "  -D <pkg> dump dependencies of <pkg>" << endl;
   cerr << "" << endl;
   return exit_r;
@@ -62,6 +64,29 @@ void tableOut( const std::string & s1 = std::string(),
   message << endl;
 }
 
+struct PQSort
+{
+  // std::less semantic
+  bool operator()( const PoolQuery::const_iterator & lhs, const PoolQuery::const_iterator & rhs ) const
+  {
+    {
+      bool l = lhs->isSystem();
+      bool r = rhs->isSystem();
+      if ( l != r )
+	return r;
+    }
+    {
+      std::string l( lhs->ident().asString() );
+      std::string r( rhs->ident().asString() );
+      if ( l != r )
+	return l < r;
+    }
+    return avo( PoolItem(*lhs), PoolItem(*rhs) );
+    return lhs->id() > rhs->id();
+  }
+
+  ui::SelectableTraits::AVOrder avo;
+};
 
 ///////////////////////////////////////////////////////////////////
 
@@ -143,7 +168,12 @@ int main( int argc, char * argv[] )
   else if ( TestSetup::isTestSetup( sysRoot ) )
   {
     message << str::form( "*** Load TestSetup from '%s'", sysRoot.c_str() ) << endl;
-    TestSetup test( sysRoot, Arch_x86_64 );
+    const char * astr = getenv( "ZYPP_TESTSUITE_FAKE_ARCH" );
+    if ( !astr || !*astr )
+      astr = getenv( "ZYPP_ARCH" );
+    if ( !astr || !*astr )
+      astr = "x86_64";
+    TestSetup test( sysRoot, Arch( astr ) );
     test.loadRepos();
     dumpRange( message, satpool.reposBegin(), satpool.reposEnd() ) << endl;
   }
@@ -195,6 +225,8 @@ int main( int argc, char * argv[] )
   ///////////////////////////////////////////////////////////////////
 
   bool ignorecase	( true );
+  bool matechexact	( false );
+  bool withSrcPackages	( false );
   bool names		( true );
   bool provides		( false );
   bool requires		( false );
@@ -204,6 +236,7 @@ int main( int argc, char * argv[] )
   bool supplements	( false );
   bool enhacements	( false );
 
+
   for ( ; argc; --argc,++argv )
   {
     if ( (*argv)[0] == '-' )
@@ -223,6 +256,7 @@ int main( int argc, char * argv[] )
 	  break;
         case 'i': ignorecase =	true;	break;
         case 'I': ignorecase =	false;	break;
+        case 'x': matechexact =	true;	break;
         case 'n': names =	true;	break;
         case 'N': names =	false;	break;
         case 'r': requires =	true;	break;
@@ -271,27 +305,50 @@ int main( int argc, char * argv[] )
       else
 	q.addString( qstr );
 
-      q.setMatchRegex();
+      if ( matechexact )
+	q.setMatchGlob();
+      else
+	q.setMatchRegex();
       q.setCaseSensitive( ! ignorecase );
 
       if ( names )
 	q.addAttribute( sat::SolvAttr::name );
       if ( provides )
+      {
 	q.addDependency( sat::SolvAttr::provides );
+	q.addDependency( sat::SolvAttr::provides, Capability(qstr) );
+      }
       if ( requires )
+      {
 	q.addDependency( sat::SolvAttr::requires );
+	q.addDependency( sat::SolvAttr::requires, Capability(qstr) );
+      }
       if ( conflicts )
+      {
 	q.addDependency( sat::SolvAttr::conflicts );
+	q.addDependency( sat::SolvAttr::conflicts, Capability(qstr) );
+      }
       if ( obsoletes )
+      {
 	q.addDependency( sat::SolvAttr::obsoletes );
+	q.addDependency( sat::SolvAttr::obsoletes, Capability(qstr) );
+      }
       if ( recommends )
+      {
 	q.addDependency( sat::SolvAttr::recommends );
+	q.addDependency( sat::SolvAttr::recommends, Capability(qstr) );
+      }
       if ( supplements )
+      {
 	q.addDependency( sat::SolvAttr::supplements );
+	q.addDependency( sat::SolvAttr::supplements, Capability(qstr) );
+      }
       if ( enhacements )
       {
 	q.addDependency( sat::SolvAttr::enhances );
+	q.addDependency( sat::SolvAttr::enhances, Capability(qstr) );
 	q.addDependency( sat::SolvAttr::suggests );
+	q.addDependency( sat::SolvAttr::suggests, Capability(qstr) );
       }
     }
 
@@ -299,8 +356,15 @@ int main( int argc, char * argv[] )
     << (conflicts?'c':'_') << (obsoletes?'o':'_') << (recommends?'m':'_') << (supplements?'s':'_') << (enhacements?'e':'_')
     << "] {" << endl;
 
+    std::set<PoolQuery::const_iterator,PQSort> qsorted;
     for_( it, q.begin(), q.end() )
+      qsorted.insert( it );
+
+    for ( auto && it : qsorted )
     {
+      if ( it->isKind( ResKind::srcpackage ) && !withSrcPackages )
+	continue;
+
       tableOut( str::numstring( it->id() ), it->asString(),
 		str::form( "(%d)%s", it->repository().info().priority(), it->repository().name().c_str() ),
 		str::numstring( PoolItem(*it)->buildtime() ) );
