@@ -27,9 +27,13 @@
 #include "zypp/base/String.h"
 #include "zypp/base/Gettext.h"
 #include "zypp/ExternalProgram.h"
+#include "zypp/base/CleanerThread_p.h"
 
 using namespace std;
 
+#undef  ZYPP_BASE_LOGGER_LOGGROUP
+#define ZYPP_BASE_LOGGER_LOGGROUP "zypp::exec"
+
 namespace zypp {
 
     ExternalProgram::ExternalProgram()
@@ -164,16 +168,22 @@ namespace zypp {
 
 
     ExternalProgram::~ExternalProgram()
-    {}
+    {
+      if ( running() ) {
+        // we got destructed while the external process is still alive
+        // make sure the zombie is cleaned up once it exits
+        CleanerThread::watchPID( pid );
+      }
+    }
 
 
 
-    void ExternalProgram::start_program( const char *const *argv,
+    void ExternalProgram::start_program(const char *const *argv,
 					 const Environment & environment,
 					 Stderr_Disposition stderr_disp,
 					 int stderr_fd,
 					 bool default_locale,
-					 const char * root )
+					 const char * root , bool switch_pgid)
     {
       pid = -1;
       _exitStatus = 0;
@@ -297,6 +307,8 @@ namespace zypp {
     	}
     	else
     	{
+            if ( switch_pgid )
+              setpgid( 0, 0);
     	    renumber_fd (to_external[0], 0); // set new stdin
     	    ::close(from_external[0]);	  // Belongs to father process
 
