@@ -23,6 +23,7 @@
 #include "zypp/Pathname.h"
 #include "zypp/CheckSum.h"
 #include "zypp/OnMediaLocation.h"
+#include "zypp/ManagedFile.h"
 
 ///////////////////////////////////////////////////////////////////
 namespace zypp
@@ -171,6 +172,43 @@ namespace zypp
        */
       Pathname provideFile(const Pathname & file, unsigned media_nr = 1, ProvideFileOptions options = PROVIDE_DEFAULT );
 
+      /**
+       * Provides an optional \a file from media \a media_nr.
+       *
+       * Like \ref provideFile (NON_INTERACTIVE), but return an empty \ref Pathname
+       * rather than throwing a \ref MediaException if the file is not present on
+       * the media.
+       */
+      Pathname provideOptionalFile( const Pathname & file, unsigned media_nr = 1 );
+
+      /**
+       * Provides \a file from \a url.
+       *
+       * \param absolute url to the file
+       * \return local pathname of the requested file
+       *
+       * \note interaction with the user does not ocurr if
+       * \ref ProvideFileOptions::NON_INTERACTIVE is set.
+       *
+       * \throws MediaException if a problem occured and user has chosen to
+       *         abort the operation. The calling code should take care
+       *         to quit the current operation.
+       * \throws SkipRequestException if a problem occured and user has chosen
+       *         to skip the current operation. The calling code should continue
+       *         with the next one, if possible.
+       * \see zypp::media::MediaManager::provideFile()
+       */
+      static ManagedFile provideFileFromUrl( const Url & file_url, ProvideFileOptions options = PROVIDE_DEFAULT );
+
+      /**
+       * Provides an optional \a file from \a url.
+       *
+       * Like \ref provideFileFromUrl( NON_INTERACTIVE ), but return an empty \ref Pathname
+       * rather than throwing a \ref MediaException if the file is not present on
+       * the media.
+       */
+      static ManagedFile provideOptionalFileFromUrl( const Url & file_url );
+
       /**
        * Release file from media.
        * This signal that file is not needed anymore.
@@ -189,6 +227,36 @@ namespace zypp
        */
       void releaseFile(const Pathname & file, unsigned media_nr = 1 );
 
+      ///////////////////////////////////////////////////////////////////
+      /// \class MediaSetAccess::ReleaseFileGuard
+      /// \brief Release a provided file upon destruction.
+      /// In case you don't want to wait until the \ref MediaSetAccess
+      /// itself goes out of scope.
+      /// \code
+      ///   MediaSetAccess media;
+      ///   OnMediaLocation loc;
+      ///   {
+      ///   	Pathname file = media.provideFile( loc );
+      ///   	ReleaseFileGuard guard( media, loc );
+      ///   }   // provided file is released here.
+      /// \endcode
+      /// \ingroup g_RAII
+      ///////////////////////////////////////////////////////////////////
+      struct ReleaseFileGuard
+      {
+	NON_COPYABLE( ReleaseFileGuard );
+	NON_MOVABLE( ReleaseFileGuard );
+	ReleaseFileGuard( MediaSetAccess & media_r, const OnMediaLocation & loc_r )
+	: _media( media_r )
+	, _loc( loc_r )
+	{}
+	~ReleaseFileGuard()
+	{ _media.releaseFile( _loc ); }
+      private:
+	MediaSetAccess & _media;
+	const OnMediaLocation & _loc;
+      };
+
       /**
        * Provides direcotry \a dir from media number \a media_nr.
        *
