@@ -179,6 +179,13 @@ namespace zypp
      * match, it's also suitable for sub-structure (flexarray) inspection
      * (\see \ref sat::LookupAttr::iterator::solvAttrSubEntry).
      *
+     * (bsc#1035729) If SolvAttr::name searches for an explicit \c kind:name,
+     * this \c kind is stored in \ref kindPredicate and will overwrite any
+     * 'global' kind restriction applied via \ref PoolQuery::addKind. This
+     * task can't be passed off to a predicate, as \ref PoolQueryMatcher::isAMatch
+     * must accept only explicit-kind-checking predicate matches, in case the
+     * 'global' kind restriction woudl otherwise discard the match.
+     *
      * \note: \see \ref addPredicate for further constraints.
      */
     struct AttrMatchData
@@ -191,6 +198,10 @@ namespace zypp
       AttrMatchData()
       {}
 
+      AttrMatchData( sat::SolvAttr attr_r )
+        : attr( attr_r )
+      {}
+
       AttrMatchData( sat::SolvAttr attr_r, const StrMatcher & strMatcher_r )
         : attr( attr_r )
         , strMatcher( strMatcher_r )
@@ -218,7 +229,7 @@ namespace zypp
 
       /** Dumb serialization.
        * \code
-       *   AttrMatchData ATTRIBUTE SEARCHSTRING [C|X] SERIALIZED_PREDICATE
+       *   AttrMatchData ATTRIBUTE SEARCHSTRING [C|SEARCHMODE|X] SERIALIZED_PREDICATE
        * \endcode
       */
       std::string serialize() const
@@ -226,10 +237,7 @@ namespace zypp
         std::string ret( "AttrMatchData" );
         str::appendEscaped( ret, attr.asString() );
         str::appendEscaped( ret, strMatcher.searchstring() );
-        // TODO: Actually the flag should be serialized too, but for PoolQuery
-        // it's by now sufficient to differ between mode OTHER and others,
-        // i.e. whether to compile or not compile.
-        str::appendEscaped( ret, strMatcher.flags().mode() == Match::OTHER ? "C" : "X" );
+        str::appendEscaped( ret, serializeMode( strMatcher.flags().mode() ) );
         str::appendEscaped( ret, predicateStr );
         return ret;
       }
@@ -249,8 +257,8 @@ namespace zypp
         AttrMatchData ret;
         ret.attr = sat::SolvAttr( words[1] );
         ret.strMatcher = StrMatcher( words[2] );
-        if ( words[3] == "C" )
-          ret.strMatcher.setFlags( Match::OTHER );
+        if ( Match::Mode mode = deserializeMode( words[3] ) )
+          ret.strMatcher.setFlags( mode );
         ret.predicateStr = words[4];
 
         // now the predicate
@@ -301,15 +309,68 @@ namespace zypp
      }
 
       sat::SolvAttr    attr;
-      StrMatcher strMatcher;
+      StrMatcher       strMatcher;
       Predicate        predicate;
       std::string      predicateStr;
+      ResKind          kindPredicate = ResKind::nokind;	// holds the 'kind' part if SolvAttr:name looks for an explicit 'kind:name'
+
+    private:
+      /** Serialize \ref strMatcher \ref Match::Mode */
+      static std::string serializeMode( Match::Mode mode_r )
+      {
+	// Legacy code used "[C|X]" to differ just between OTHER (need to (C)ompile) and
+	// using the default search mode. As we now allow to specify a SEARCHMODE we
+	// need to serialize it:
+	switch ( mode_r )
+	{
+#define OUTS(M,S) case Match::M: return #S; break
+	  // (C)ompile
+	  OUTS( OTHER,		C );
+	  // well known modes:
+	  OUTS( STRING,		T );
+	  OUTS( STRINGSTART,	S );
+	  OUTS( STRINGEND,	E );
+	  OUTS( SUBSTRING,	B );
+	  OUTS( GLOB,		G );
+	  OUTS( REGEX,		R );
+#undef OUTS
+	  // everything else use default
+	  case Match::NOTHING:
+	    break;
+	}
+	return "X";
+      }
+
+      /** Deserialize \ref strMatcher \ref Match::Mode */
+      static Match::Mode deserializeMode( const std::string & str_r )
+      {
+	switch ( str_r[0] )
+	{
+#define OUTS(M,C) case *#C: return Match::M; break
+	  // (C)ompile
+	  OUTS( OTHER,		C );
+	  // well known modes:
+	  OUTS( STRING,		T );
+	  OUTS( STRINGSTART,	S );
+	  OUTS( STRINGEND,	E );
+	  OUTS( SUBSTRING,	B );
+	  OUTS( GLOB,		G );
+	  OUTS( REGEX,		R );
+#undef OUTS
+	  // everything else use default
+	  default:
+	    break;
+	}
+	return Match::NOTHING;
+      }
     };
 
     /** \relates AttrMatchData */
     inline std::ostream & operator<<( std::ostream & str, const AttrMatchData & obj )
     {
       str << obj.attr << ": " << obj.strMatcher;
+      if ( obj.kindPredicate )
+        str << " +(" << obj.kindPredicate << ")";
       if ( obj.predicate )
         str << " +(" << obj.predicateStr << ")";
       return str;
@@ -399,6 +460,24 @@ namespace zypp
 
   public:
 
+    bool operator<( const PoolQuery::Impl & rhs ) const
+    {
+#define OUTS(A) if ( A != rhs.A ) return A < rhs.A;
+      OUTS( _strings );
+      OUTS( _attrs );
+      OUTS( _uncompiledPredicated );
+      OUTS( _flags.get() );
+      OUTS( _match_word );
+      OUTS( _require_all );
+      OUTS( _status_flags );
+      OUTS( _edition );
+      OUTS( _op.inSwitch() );
+      OUTS( _repos );
+      OUTS( _kinds );
+#undef OUTS
+      return false;
+    }
+
     bool operator==( const PoolQuery::Impl & rhs ) const
     {
       if ( _flags == rhs._flags
@@ -620,9 +699,10 @@ attremptycheckend:
           if ( joined.size() > 1 ) // switch to regex for multiple strings
             cflags.setModeRegex();
 
-          _attrMatchList.push_back( AttrMatchData( it->attr,
-                                    StrMatcher( rcstrings, cflags ),
-                                                      it->predicate, it->predicateStr ) );
+	  // copy and exchange the StrMatcher
+	  AttrMatchData nattr( *it );
+	  nattr.strMatcher = StrMatcher( rcstrings, cflags ),
+          _attrMatchList.push_back( std::move(nattr) );
         }
         else
         {
@@ -855,30 +935,48 @@ attremptycheckend:
   { _pimpl->_attrs[attr].insert(value); }
 
   void PoolQuery::addDependency( const sat::SolvAttr & attr, const std::string & name, const Rel & op, const Edition & edition )
-  { return addDependency( attr, name, op, edition, Arch_empty ); }
+  {
+    // Default Match::OTHER indicates need to compile, i.e. to merge name into the global search string and mode.
+    return addDependency( attr, name, op, edition, Arch_empty, Match::OTHER );
+  }
 
   void PoolQuery::addDependency( const sat::SolvAttr & attr, const std::string & name, const Rel & op, const Edition & edition, const Arch & arch )
   {
-    switch ( op.inSwitch() )
+    // Default Match::OTHER indicates need to compile, i.e. to merge name into the global search string and mode.
+    return addDependency( attr, name, op, edition, arch, Match::OTHER );
+  }
+
+  void PoolQuery::addDependency( const sat::SolvAttr & attr, const std::string & name, const Rel & op, const Edition & edition, const Arch & arch, Match::Mode mode )
+  {
+    if ( op == Rel::NONE )	// will never match.
+      return;
+
+    // SolvAttr::name with explicit 'kind:name' will overwrite the default _kinds
+    ResKind explicitKind;
+    if ( attr == sat::SolvAttr::name )
+      explicitKind = ResKind::explicitBuiltin( name );
+
+    // Legacy: Match::OTHER and no additional constraints on edition/arch/kind
+    //         require addAttribute, otherwise de-serialisation breaks (serialized
+    //         and de-serialized query could be !=).
+    // From the results POV we could also use the predicated case below.
+    if ( op == Rel::ANY && arch.empty() && !explicitKind && mode == Match::OTHER )
     {
-      case Rel::ANY_e:	// no additional constraint on edition.
-        if ( arch.empty() )	// no additional constraint on arch.
-	{
-	  addAttribute( attr, name );
-	  return;
-	}
-	break;
-
-      case Rel::NONE_e:	// will never match.
-        return;
-
-      default: // go and add the predicated query (uncompiled)
-        break;
+      addAttribute( attr, name );
+      return;
     }
 
     // Match::OTHER indicates need to compile
     // (merge global search strings into name).
-    AttrMatchData attrMatchData( attr, StrMatcher( name, Match::OTHER ) );
+    AttrMatchData attrMatchData( attr );
+    if ( !explicitKind )
+      attrMatchData.strMatcher = StrMatcher( name, mode );
+    else
+    {
+      // ResKind::explicitBuiltin call above asserts the presence of the ':' in name
+      attrMatchData.strMatcher = StrMatcher( strchr( name.c_str(), ':')+1, mode );
+      attrMatchData.kindPredicate = explicitKind;
+    }
 
     if ( isDependencyAttribute( attr ) )
       attrMatchData.addPredicate( EditionRangePredicate( op, edition, arch ) );
@@ -1409,6 +1507,9 @@ attremptycheckend:
   bool PoolQuery::operator==( const PoolQuery & rhs ) const
   { return *_pimpl == *rhs._pimpl; }
 
+  bool PoolQuery::operator<( const PoolQuery & rhs ) const
+  { return *_pimpl < *rhs._pimpl; }
+
   ///////////////////////////////////////////////////////////////////
   namespace detail
   { /////////////////////////////////////////////////////////////////
@@ -1614,19 +1715,18 @@ attremptycheckend:
 	  }
 	  /////////////////////////////////////////////////////////////////////
 	  sat::Solvable inSolvable( base_r.inSolvable() );
-	  // Kind restriction:
-	  if ( ! _kinds.empty() && ! inSolvable.isKind( _kinds.begin(), _kinds.end() ) )
-	  {
-            base_r.nextSkipSolvable();
-            return false;
-	  }
-
 	  // Edition restriction:
 	  if ( _op != Rel::ANY && !compareByRel( _op, inSolvable.edition(), _edition, Edition::Match() ) )
 	  {
 	    base_r.nextSkipSolvable();
 	    return false;
 	  }
+
+	  // Kind restriction:
+	  // Delay the decision to nextSkipSolvable and return false, as there may be
+	  // some explicit kind:name predicate which overrules the global kinds.
+	  bool globalKindOk =( _kinds.empty() || inSolvable.isKind( _kinds.begin(), _kinds.end() ) );
+
 	  /////////////////////////////////////////////////////////////////////
 	  // string and predicate matching:
 
@@ -1634,17 +1734,38 @@ attremptycheckend:
           {
             // String matching was done by the base iterator.
             // Now check any predicate:
-            const AttrMatchData::Predicate & predicate( _attrMatchList.front().predicate );
-            if ( ! predicate || predicate( base_r ) )
+	    const AttrMatchData & matchData( _attrMatchList.front() );
+
+	    if ( matchData.kindPredicate )
+	    {
+	      if ( matchData.kindPredicate != inSolvable.kind() )
+	      {
+		base_r.nextSkipSolvable();	// this matchData will never match in this solvable
+		return false;
+	      }
+	    }
+	    else if ( !globalKindOk )
+	      return false;			// only matching kindPredicate could overwrite this
+
+            if ( !matchData.predicate || matchData.predicate( base_r ) )
               return true;
 
-            return false; // no skip as there may be more occurrences od this attr.
+            return false; // no skip as there may be more occurrences in this solvable of this attr.
           }
 
           // Here: search all attributes ;(
           for_( mi, _attrMatchList.begin(), _attrMatchList.end() )
           {
             const AttrMatchData & matchData( *mi );
+
+	    if ( matchData.kindPredicate )
+	    {
+	      if ( matchData.kindPredicate != inSolvable.kind() )
+		continue;			// this matchData does not apply
+	    }
+	    else if ( !globalKindOk )
+	      continue;				// only matching kindPredicate could overwrite this
+
             sat::LookupAttr q( matchData.attr, inSolvable );
             if ( matchData.strMatcher ) // an empty searchstring matches always
               q.setStrMatcher( matchData.strMatcher );
