@@ -287,6 +287,8 @@ namespace zypp
     void addDependency( const sat::SolvAttr & attr, const std::string & name, const Rel & op, const Edition & edition );
     /**  \overload also restricting architecture */
     void addDependency( const sat::SolvAttr & attr, const std::string & name, const Rel & op, const Edition & edition, const Arch & arch );
+    /**  \overload also use an individual Match::Mode for the \a name */
+    void addDependency( const sat::SolvAttr & attr, const std::string & name, const Rel & op, const Edition & edition, const Arch & arch, Match::Mode mode );
 
     /** \overload Query <tt>"name|global == edition"</tt>. */
     void addDependency( const sat::SolvAttr & attr, const std::string & name, const Edition & edition )
@@ -463,6 +465,7 @@ namespace zypp
     /** Return a human-readable description of the query */
     std::string asString() const;
 
+    bool operator<(const PoolQuery& b) const;
     bool operator==(const PoolQuery& b) const;
     bool operator!=(const PoolQuery& b) const { return !(*this == b ); }
 
