@@ -24,20 +24,167 @@
 #include "zypp/base/Exception.h"
 #include "zypp/base/LogTools.h"
 #include "zypp/Date.h"
+#include "zypp/KeyManager.h"
 
-/** \todo Fix duplicate define in PublicKey/KeyRing */
-#define GPG_BINARY "/usr/bin/gpg2"
+#include <gpgme.h>
 
 using std::endl;
 
+#undef  ZYPP_BASE_LOGGER_LOGGROUP
+#define ZYPP_BASE_LOGGER_LOGGROUP "zypp::gpg"
+
 ///////////////////////////////////////////////////////////////////
 namespace zypp
-{ /////////////////////////////////////////////////////////////////
+{
+  ///////////////////////////////////////////////////////////////////
+  namespace
+  {
+    inline bool isExpired( const Date & expires_r )
+    { return( expires_r && expires_r < Date::now() ); }
+
+    inline int hasDaysToLive( const Date & expires_r )
+    {
+      if ( expires_r )
+      {
+	Date exp( expires_r - Date::now() );
+	int ret = exp / Date::day;
+	if ( exp < 0 ) ret -= 1;
+	return ret;
+      }
+      return INT_MAX;
+    }
+
+    inline std::string expiresDetail( const Date & expires_r )
+    {
+      str::Str str;
+      if ( ! expires_r )
+      {
+	// translators: an annotation to a gpg keys expiry date
+	str << _("does not expire");
+      }
+      else if ( isExpired( expires_r ) )
+      {
+	// translators: an annotation to a gpg keys expiry date: "expired: 1999-04-12"
+	str << ( str::Format(_("expired: %1%") ) % expires_r.printDate() );
+      }
+      else
+      {
+	// translators: an annotation to a gpg keys expiry date: "expires: 2111-04-12"
+	str << ( str::Format(_("expires: %1%") ) % expires_r.printDate() );
+      }
+      return str;
+    }
+
+    inline std::string expiresDetailVerbose( const Date & expires_r )
+    {
+      if ( !expires_r )
+      { // translators: an annotation to a gpg keys expiry date
+	return _("(does not expire)");
+      }
+      std::string ret( expires_r.asString() );
+      int ttl( hasDaysToLive( expires_r ) );
+      if ( ttl <= 90 )
+      {
+	ret += " ";
+	if ( ttl < 0 )
+	{ // translators: an annotation to a gpg keys expiry date
+	  ret += _("(EXPIRED)");
+	}
+	else if ( ttl == 0 )
+	{ // translators: an annotation to a gpg keys expiry date
+	  ret += _("(expires within 24h)");
+	}
+	else
+	{ // translators: an annotation to a gpg keys expiry date
+	  ret += str::form( PL_("(expires in %d day)", "(expires in %d days)", ttl ), ttl );
+	}
+      }
+      return ret;
+    }
+
+  } //namespace
+  ///////////////////////////////////////////////////////////////////
+
+
+  ///////////////////////////////////////////////////////////////////
+  /// \class PublicSubkeyData::Impl
+  /// \brief  PublicSubkeyData implementation.
+  ///////////////////////////////////////////////////////////////////
+
+  struct PublicSubkeyData::Impl
+  {
+    std::string _id;
+    Date        _created;
+    Date        _expires;
+
+  public:
+    /** Offer default Impl. */
+    static shared_ptr<Impl> nullimpl();
+
+  private:
+    friend Impl * rwcowClone<Impl>( const Impl * rhs );
+    /** clone for RWCOW_pointer */
+    Impl * clone() const;
+  };
+
+  shared_ptr<zypp::PublicSubkeyData::Impl> PublicSubkeyData::Impl::nullimpl()
+  {
+    static shared_ptr<Impl> _nullimpl( new Impl );
+    return _nullimpl;
+  }
+
+  zypp::PublicSubkeyData::Impl *PublicSubkeyData::Impl::clone() const
+  {
+    return new Impl( *this );
+  }
+
+  ///////////////////////////////////////////////////////////////////
+  /// class PublicSubkeyData
+  ///////////////////////////////////////////////////////////////////
+
+  PublicSubkeyData::PublicSubkeyData()
+    : _pimpl( Impl::nullimpl() )
+  {}
+
+  PublicSubkeyData::PublicSubkeyData(const _gpgme_subkey *rawSubKeyData)
+    : _pimpl (new Impl)
+  {
+    _pimpl->_created = zypp::Date(rawSubKeyData->timestamp);
+    _pimpl->_expires = zypp::Date(rawSubKeyData->expires);
+    _pimpl->_id = str::asString(rawSubKeyData->keyid);
+  }
+
+  PublicSubkeyData::~PublicSubkeyData()
+  {}
+
+  PublicSubkeyData::operator bool() const
+  { return !_pimpl->_id.empty(); }
+
+  std::string PublicSubkeyData::id() const
+  { return _pimpl->_id; }
+
+  Date PublicSubkeyData::created() const
+  { return _pimpl->_created; }
+
+  Date PublicSubkeyData::expires() const
+  { return _pimpl->_expires; }
+
+  bool PublicSubkeyData::expired() const
+  { return isExpired( _pimpl->_expires ); }
+
+  int PublicSubkeyData::daysToLive() const
+  { return hasDaysToLive( _pimpl->_expires ); }
+
+  std::string PublicSubkeyData::asString() const
+  {
+    return str::Str() << id() << " " << created().printDate() << " [" << expiresDetail( expires() ) << "]";
+  }
 
   ///////////////////////////////////////////////////////////////////
   /// \class PublicKeyData::Impl
   /// \brief  PublicKeyData implementation.
   ///////////////////////////////////////////////////////////////////
+  ///
   struct PublicKeyData::Impl
   {
     std::string _id;
@@ -46,21 +193,80 @@ namespace zypp
     Date        _created;
     Date        _expires;
 
+    std::vector<PublicSubkeyData> _subkeys;
+
+  public:
+    bool hasSubkeyId( const std::string & id_r ) const;
+
   public:
     /** Offer default Impl. */
-    static shared_ptr<Impl> nullimpl()
-    {
-      static shared_ptr<Impl> _nullimpl( new Impl );
-      return _nullimpl;
-    }
+    static shared_ptr<Impl> nullimpl();
+    static shared_ptr<Impl> fromGpgmeKey(gpgme_key_t rawData);
 
   private:
     friend Impl * rwcowClone<Impl>( const Impl * rhs );
     /** clone for RWCOW_pointer */
-    Impl * clone() const
-    { return new Impl( *this ); }
+    Impl * clone() const;
   };
-  ///////////////////////////////////////////////////////////////////
+
+  bool PublicKeyData::Impl::hasSubkeyId(const std::string &id_r) const
+  {
+    bool ret = false;
+    for ( const PublicSubkeyData & sub : _subkeys )
+    {
+      if ( sub.id() == id_r )
+      {
+        ret = true;
+        break;
+      }
+    }
+    return ret;
+  }
+
+  shared_ptr<PublicKeyData::Impl> PublicKeyData::Impl::nullimpl()
+  {
+    static shared_ptr<Impl> _nullimpl( new Impl );
+    return _nullimpl;
+  }
+
+  shared_ptr<PublicKeyData::Impl> PublicKeyData::Impl::fromGpgmeKey(gpgme_key_t rawData)
+  {
+    //gpgpme stores almost nothing in the top level key
+    //the information we look for is stored in the subkey, where subkey[0]
+    //is always the primary key
+    gpgme_subkey_t sKey = rawData->subkeys;
+    if (sKey) {
+      shared_ptr<PublicKeyData::Impl> data(new Impl);
+      //libzypp expects the date of the first signature on the first uid
+      if(rawData->uids && rawData->uids->signatures)
+        data->_created = zypp::Date(rawData->uids->signatures->timestamp);
+      else
+        data->_created = zypp::Date(sKey->timestamp);
+
+      data->_expires = zypp::Date(sKey->expires);
+      data->_fingerprint = str::asString(sKey->fpr);
+      data->_id = str::asString(sKey->keyid);
+
+      //get the primary user ID
+      if (rawData->uids) {
+        data->_name = str::asString(rawData->uids->uid);
+      }
+
+      //the rest of the keys
+      sKey = sKey->next;
+      while (sKey) {
+        data->_subkeys.push_back( PublicSubkeyData(sKey) );
+        sKey = sKey->next;
+      }
+      return data;
+    }
+    return nullimpl();
+  }
+
+  zypp::PublicKeyData::Impl *PublicKeyData::Impl::clone() const
+  {
+    return new Impl( *this );
+  }
 
   ///////////////////////////////////////////////////////////////////
   /// class PublicKeyData
@@ -70,9 +276,16 @@ namespace zypp
     : _pimpl( Impl::nullimpl() )
   {}
 
+  PublicKeyData::PublicKeyData(shared_ptr<Impl> data)
+    : _pimpl( data )
+  {}
+
   PublicKeyData::~PublicKeyData()
   {}
 
+  PublicKeyData PublicKeyData::fromGpgmeKey(_gpgme_key *data)
+  { return PublicKeyData(Impl::fromGpgmeKey(data)); }
+
   PublicKeyData::operator bool() const
   { return !_pimpl->_fingerprint.empty(); }
 
@@ -92,46 +305,13 @@ namespace zypp
   { return _pimpl->_expires; }
 
   bool PublicKeyData::expired() const
-  { return( _pimpl->_expires && _pimpl->_expires < Date::now() ); }
+  { return isExpired( _pimpl->_expires ); }
 
   int PublicKeyData::daysToLive() const
-  {
-    if ( _pimpl->_expires )
-    {
-      Date exp( _pimpl->_expires - Date::now() );
-      int ret = exp / Date::day;
-      if ( exp < 0 ) ret -= 1;
-      return ret;
-    }
-    return INT_MAX;
-  }
+  { return hasDaysToLive( _pimpl->_expires ); }
 
   std::string PublicKeyData::expiresAsString() const
-  {
-    if ( !_pimpl->_expires )
-    { // translators: an annotation to a gpg keys expiry date
-      return _("(does not expire)");
-    }
-    std::string ret( _pimpl->_expires.asString() );
-    int ttl( daysToLive() );
-    if ( ttl <= 90 )
-    {
-      ret += " ";
-      if ( ttl < 0 )
-      { // translators: an annotation to a gpg keys expiry date
-	ret += _("(EXPIRED)");
-      }
-      else if ( ttl == 0 )
-      { // translators: an annotation to a gpg keys expiry date
-	ret += _("(expires within 24h)");
-      }
-      else
-      { // translators: an annotation to a gpg keys expiry date
-	ret += str::form( PL_("(expires in %d day)", "(expires in %d days)", ttl ), ttl );
-      }
-    }
-    return ret;
-  }
+  { return expiresDetailVerbose( _pimpl->_expires ); }
 
   std::string PublicKeyData::gpgPubkeyVersion() const
   { return _pimpl->_id.empty() ? _pimpl->_id : str::toLower( _pimpl->_id.substr(8,8) ); }
@@ -139,16 +319,34 @@ namespace zypp
   std::string PublicKeyData::gpgPubkeyRelease() const
   { return _pimpl->_created ? str::hexstring( _pimpl->_created ).substr(2) : std::string(); }
 
+  std::string PublicKeyData::rpmName() const
+  { return str::Format( "gpg-pubkey-%1%-%2%" ) % gpgPubkeyVersion() % gpgPubkeyRelease();  }
+
   std::string PublicKeyData::asString() const
   {
-    return str::form( "[%s-%s] [%s] [%s] [TTL %d]",
-		      _pimpl->_id.c_str(),
-		      gpgPubkeyRelease().c_str(),
-		      _pimpl->_name.c_str(),
-		      _pimpl->_fingerprint.c_str(),
-		      daysToLive() );
+    str::Str str;
+    str << "[" << _pimpl->_id << "-" << gpgPubkeyRelease();
+    for ( auto && sub : _pimpl->_subkeys )
+      str << ", " << sub.id();
+    return str << "] [" << _pimpl->_name.c_str() << "] [" << expiresDetail( _pimpl->_expires ) << "]";
   }
 
+  bool PublicKeyData::hasSubkeys() const
+  { return !_pimpl->_subkeys.empty(); }
+
+  Iterable<PublicKeyData::SubkeyIterator> PublicKeyData::subkeys() const
+  { return makeIterable( &(*_pimpl->_subkeys.begin()), &(*_pimpl->_subkeys.end()) ); }
+
+  bool PublicKeyData::providesKey( const std::string & id_r ) const
+  {
+    if ( id_r.size() == 8 )	// as a convenience allow to test the 8byte short ID rpm uses as gpg-pubkey version
+      return str::endsWithCI( _pimpl->_id, id_r );
+    return( id_r == _pimpl->_id || _pimpl->hasSubkeyId( id_r ) );
+  }
+
+  PublicKeyData::AsciiArt PublicKeyData::asciiArt() const
+  { return AsciiArt( fingerprint() /* TODO: key algorithm could be added as top tile. */ ); }
+
   std::ostream & dumpOn( std::ostream & str, const PublicKeyData & obj )
   {
     str << "[" << obj.name() << "]" << endl;
@@ -157,8 +355,9 @@ namespace zypp
     str << "  cre " << Date::ValueType(obj.created()) << ' ' << obj.created() << endl;
     str << "  exp " << Date::ValueType(obj.expires()) << ' ' << obj.expiresAsString() << endl;
     str << "  ttl " << obj.daysToLive() << endl;
+    for ( auto && sub : obj._pimpl->_subkeys )
+      str << "  sub " << sub << endl;
     str << "  rpm " << obj.gpgPubkeyVersion() << "-" << obj.gpgPubkeyRelease() << endl;
-    str << "]";
     return str;
   }
 
@@ -166,132 +365,6 @@ namespace zypp
   { return ( lhs.fingerprint() == rhs.fingerprint() && lhs.created() == rhs.created() ); }
 
 
-  ///////////////////////////////////////////////////////////////////
-  /// \class PublicKeyScanner::Impl
-  /// \brief  PublicKeyScanner implementation.
-  ///////////////////////////////////////////////////////////////////
-  struct PublicKeyScanner::Impl
-  {
-    std::vector<std::string>			_words;
-    enum { pNONE, pPUB, pSIG, pFPR, pUID }	_parseEntry;
-    bool 					_parseOff;	// no 'sub:' key parsing
-
-   Impl()
-      : _parseEntry( pNONE )
-      , _parseOff( false )
-    {}
-
-    void scan( std::string & line_r, std::list<PublicKeyData> & keys_r )
-    {
-      // pub:-:1024:17:A84EDAE89C800ACA:971961473:1214043198::-:SuSE Package Signing Key <build@suse.de>:
-      // fpr:::::::::79C179B2E1C820C1890F9994A84EDAE89C800ACA:
-      // sig:::17:A84EDAE89C800ACA:1087899198:::::[selfsig]::13x:
-      // sig:::17:9E40E310000AABA4:980442706::::[User ID not found]:10x:
-      // sig:::1:77B2E6003D25D3D9:980443247::::[User ID not found]:10x:
-      // sig:::17:A84EDAE89C800ACA:1318348291:::::[selfsig]::13x:
-      // sub:-:2048:16:197448E88495160C:971961490:1214043258::: [expires: 2008-06-21]
-      // sig:::17:A84EDAE89C800ACA:1087899258:::::[keybind]::18x:
-      if ( line_r.empty() )
-	return;
-
-      // quick check for interesting entries, no parsing in subkeys
-      _parseEntry = pNONE;
-      switch ( line_r[0] )
-      {
-	case 'p':
-	  if ( line_r[1] == 'u' && line_r[2] == 'b' && line_r[3] == ':' )
-	  {
-	    _parseEntry = pPUB;
-	    _parseOff = false;
-	  }
-	  break;
-
-	case 'f':
-	  if ( line_r[1] == 'p' && line_r[2] == 'r' && line_r[3] == ':' )
-	    _parseEntry = pFPR;
-	  break;
-
-	case 'u':
-	  if ( line_r[1] == 'i' && line_r[2] == 'd' && line_r[3] == ':' )
-	    _parseEntry = pUID;
-	  break;
-
-	case 's':
-	  if ( line_r[1] == 'i' && line_r[2] == 'g' && line_r[3] == ':' )
-	    _parseEntry = pSIG;
-	  else if ( line_r[1] == 'u' && line_r[2] == 'b' && line_r[3] == ':' )
-	    _parseOff = true;
-	  break;
-
-	default:
-	  return;
-      }
-      if ( _parseOff || _parseEntry == pNONE )
-	return;
-
-      if ( line_r[line_r.size()-1] == '\n' )
-	line_r.erase( line_r.size()-1 );
-      // DBG << line_r << endl;
-
-      _words.clear();
-      str::splitFields( line_r, std::back_inserter(_words), ":" );
-
-      PublicKeyData * key( &keys_r.back() );
-
-      switch ( _parseEntry )
-      {
-	case pPUB:
-	  keys_r.push_back( PublicKeyData() );	// reset upon new key
-	  key = &keys_r.back();
-	  key->_pimpl->_id      = _words[4];
-	  key->_pimpl->_name    = str::replaceAll( _words[9], "\\x3a", ":" );
-	  key->_pimpl->_created = Date(str::strtonum<Date::ValueType>(_words[5]));
-	  key->_pimpl->_expires = Date(str::strtonum<Date::ValueType>(_words[6]));
-	  break;
-
-	case pSIG:
-	  // Update creation/modification date from signatures type "13x".
-	  if ( ( _words.size() > 10 && _words[10] == "13x" )
-	    || ( _words.size() > 12 && _words[12] == "13x" ) )
-	  {
-	    Date cdate(str::strtonum<Date::ValueType>(_words[5]));
-	    if ( key->_pimpl->_created < cdate )
-	      key->_pimpl->_created = cdate;
-	  }
-	  break;
-
-	case pFPR:
-	  if ( key->_pimpl->_fingerprint.empty() )
-	    key->_pimpl->_fingerprint = _words[9];
-	  break;
-
-	case pUID:
-	  if ( ! _words[9].empty() )
-	    key->_pimpl->_name = str::replaceAll( _words[9], "\\x3a", ":" );
-	  break;
-
-	case pNONE:
-	  break;	// intentionally no default:
-      }
-    }
-  };
-  ///////////////////////////////////////////////////////////////////
-
-  ///////////////////////////////////////////////////////////////////
-  // class PublicKeyScanner
-  ///////////////////////////////////////////////////////////////////
-
-  PublicKeyScanner::PublicKeyScanner()
-    : _pimpl( new Impl )
-  {}
-
-  PublicKeyScanner::~PublicKeyScanner()
-  {}
-
-  void PublicKeyScanner::scan( std::string line_r )
-  { _pimpl->scan( line_r, _keys ); }
-
-
   ///////////////////////////////////////////////////////////////////
   /// \class PublicKey::Impl
   /// \brief  PublicKey implementation.
@@ -302,6 +375,7 @@ namespace zypp
     {}
 
     Impl( const Pathname & keyFile_r )
+    : _dontUseThisPtrDirectly( new filesystem::TmpFile )
     {
       PathInfo info( keyFile_r );
       MIL << "Taking pubkey from " << keyFile_r << " of size " << info.size() << " and sha1 " << filesystem::checksum(keyFile_r, "sha1") << endl;
@@ -309,18 +383,18 @@ namespace zypp
       if ( !info.isExist() )
         ZYPP_THROW(Exception("Can't read public key from " + keyFile_r.asString() + ", file not found"));
 
-      if ( filesystem::hardlinkCopy( keyFile_r, _dataFile.path() ) != 0 )
-	ZYPP_THROW(Exception("Can't copy public key data from " + keyFile_r.asString() + " to " +  _dataFile.path().asString() ));
+      if ( filesystem::hardlinkCopy( keyFile_r, path() ) != 0 )
+	ZYPP_THROW(Exception("Can't copy public key data from " + keyFile_r.asString() + " to " +  path().asString() ));
 
       readFromFile();
     }
 
     Impl( const filesystem::TmpFile & sharedFile_r )
-      : _dataFile( sharedFile_r )
+      : _dontUseThisPtrDirectly( new filesystem::TmpFile( sharedFile_r ) )
     { readFromFile(); }
 
     Impl( const filesystem::TmpFile & sharedFile_r, const PublicKeyData & keyData_r )
-      : _dataFile( sharedFile_r )
+      : _dontUseThisPtrDirectly( new filesystem::TmpFile( sharedFile_r ) )
       , _keyData( keyData_r )
     {
       if ( ! keyData_r )
@@ -330,12 +404,16 @@ namespace zypp
       }
     }
 
+    Impl( const PublicKeyData & keyData_r )
+      : _keyData( keyData_r )
+    {}
+
     public:
       const PublicKeyData & keyData() const
       { return _keyData; }
 
       Pathname path() const
-      { return _dataFile.path(); }
+      { return( /*the one and only intended use*/_dontUseThisPtrDirectly ? _dontUseThisPtrDirectly->path() : Pathname() ); }
 
       const std::list<PublicKeyData> & hiddenKeys() const
       { return _hiddenKeys; }
@@ -346,67 +424,43 @@ namespace zypp
 
       void readFromFile()
       {
-        PathInfo info( _dataFile.path() );
+        PathInfo info( path() );
         MIL << "Reading pubkey from " << info.path() << " of size " << info.size() << " and sha1 " << filesystem::checksum(info.path(), "sha1") << endl;
 
-	static std::string tmppath( _initHomeDir() );
-	std::string datapath( _dataFile.path().asString() );
+        //@TODO is this still required? KeyManagerCtx creates a homedir on the fly
+        static std::string tmppath( _initHomeDir() );
 
-        const char* argv[] =
+        KeyManagerCtx::Ptr ctx = KeyManagerCtx::createForOpenPGP();
+        if (!ctx || !ctx->setHomedir(tmppath)) {
+          ZYPP_THROW( Exception( std::string("Can't read public key data: Setting the keyring path failed!")) );
+        }
+
+        std::list<PublicKeyData> keys = ctx->readKeyFromFile(path());
+        switch ( keys.size() )
         {
-          GPG_BINARY,
-          "-v",
-          "--no-default-keyring",
-          "--fixed-list-mode",
-          "--with-fingerprint",
-          "--with-colons",
-          "--homedir",
-          tmppath.c_str(),
-          "--quiet",
-          "--no-tty",
-          "--no-greeting",
-          "--batch",
-          "--status-fd", "1",
-          datapath.c_str(),
-          NULL
-        };
-        ExternalProgram prog( argv, ExternalProgram::Discard_Stderr, false, -1, true );
+          case 0:
+            ZYPP_THROW( BadKeyException( "File " + path().asString() + " doesn't contain public key data" , path() ) );
+            break;
 
-	PublicKeyScanner scanner;
-        for ( std::string line = prog.receiveLine(); !line.empty(); line = prog.receiveLine() )
-        {
-	  scanner.scan( line );
-	}
-        int ret = prog.close();
+          case 1:
+            // ok.
+            _keyData = keys.back();
+            _hiddenKeys.clear();
+            break;
 
-	switch ( scanner._keys.size() )
-	{
-	  case 0:
-	    if ( ret == 129 )
-	      ZYPP_THROW( Exception( std::string("Can't read public key data: ") + GPG_BINARY + " is not installed!" ) );
-	    else
-	      ZYPP_THROW( BadKeyException( "File " + _dataFile.path().asString() + " doesn't contain public key data" , _dataFile.path() ) );
-	    break;
+          default:
+            WAR << "File " << path().asString() << " contains multiple keys: " <<  keys << endl;
+            _keyData = keys.back();
+            keys.pop_back();
+            _hiddenKeys.swap( keys );
+            break;
+        }
 
-	  case 1:
-	    // ok.
-	    _keyData = scanner._keys.back();
-	    _hiddenKeys.clear();
-	    break;
-
-	  default:
-	    WAR << "File " << _dataFile.path().asString() << " contains multiple keys: " <<  scanner._keys << endl;
-	    _keyData = scanner._keys.back();
-	    scanner._keys.pop_back();
-	    _hiddenKeys.swap( scanner._keys );
-	    break;
-	}
-
-	MIL << "Read pubkey from " << info.path() << ": " << _keyData << endl;
+        MIL << "Read pubkey from " << info.path() << ": " << _keyData << endl;
       }
 
     private:
-      filesystem::TmpFile	_dataFile;
+      shared_ptr<filesystem::TmpFile> _dontUseThisPtrDirectly; // shared_ptr ok because TmpFile itself is a refernce type (no COW)
       PublicKeyData		_keyData;
       std::list<PublicKeyData>  _hiddenKeys;
 
@@ -430,7 +484,7 @@ namespace zypp
   // class PublicKey
   ///////////////////////////////////////////////////////////////////
   PublicKey::PublicKey()
-  : _pimpl( Impl::nullimpl() )
+    : _pimpl( Impl::nullimpl() )
   {}
 
   PublicKey::PublicKey( const Pathname & file )
@@ -441,8 +495,12 @@ namespace zypp
   : _pimpl( new Impl( sharedfile ) )
   {}
 
-  PublicKey::PublicKey( const filesystem::TmpFile & sharedfile, const PublicKeyData & keydata )
-  : _pimpl( new Impl( sharedfile, keydata ) )
+  PublicKey::PublicKey( const filesystem::TmpFile & sharedfile, const PublicKeyData & keyData_r )
+  : _pimpl( new Impl( sharedfile, keyData_r ) )
+  {}
+
+  PublicKey::PublicKey( const PublicKeyData & keyData_r )
+  : _pimpl( new Impl( keyData_r ) )
   {}
 
   PublicKey::~PublicKey()
@@ -490,6 +548,9 @@ namespace zypp
   std::string PublicKey::asString() const
   { return keyData().asString(); }
 
+  std::string PublicKey::rpmName() const
+  { return keyData().rpmName(); }
+
   bool PublicKey::operator==( const PublicKey & rhs ) const
   { return rhs.keyData() == keyData(); }
 
@@ -499,6 +560,9 @@ namespace zypp
   std::ostream & dumpOn( std::ostream & str, const PublicKey & obj )
   { return dumpOn( str, obj.keyData() ); }
 
+
+
+
   /////////////////////////////////////////////////////////////////
 } // namespace zypp
 ///////////////////////////////////////////////////////////////////
