@@ -235,15 +235,36 @@ namespace zypp
        */
       Pathname metadataPath() const;
       /**
-       * \short set the path where the local metadata is stored
+       * \short Set the path where the local metadata is stored
        *
-       * The path to the metadata of this repository
-       * was defined, or empty if nowhere.
+       * The path to the repositories metadata is usually provided by
+       * the RepoManager. If you want to use a temporary repository
+       * (not under RepoManagers control), and you set a metadataPath
+       * with basename \c %AUTO%, all data directories (raw metadata,
+       * solv file and package cache) will be created by replacing \c %AUTO%
+       * with \c %RAW%, \c %SLV% or \c %PKG% . This will change the value
+       * of \ref packagesPath accordingly, unless you assigned a custom
+       * value using \ref setPackagesPath.
+       *
+       * \code
+       *   RepoInfo repo;
+       *   repo.setAlias( "Temp" );
+       *   repo.setBaseUrl( Url("http://someserver/somepath/") );
+       *   repo.setMetadataPath( "/tmp/temprepodata/%AUTO%" );
+       *
+       *   // will use
+       *   //  /tmp/temprepodata/%RAW% - raw metadata
+       *   //                   /%SLV% - solv file
+       *   //                   /%PKG% - packages
+       *\endcode
        *
        * \param path directory path
        */
       void setMetadataPath( const Pathname &path );
 
+      /** Whether \ref metadataPath uses \c %AUTO% setup. */
+      bool usesAutoMethadataPaths() const;
+
       /**
        * \short Path where this repo packages are cached
        */
@@ -256,30 +277,80 @@ namespace zypp
       void setPackagesPath( const Pathname &path );
 
 
-      /** Whether default signature checking should be performed for this repo.
+      /** \name Repository gpgchecks
+       * How signature checking should be performed for this repo.
        *
-       * This will turn on \ref repoGpgCheck for signed repos and
-       * \ref pkgGpgCheck for unsigned ones or if \ref repoGpgCheck is off.
+       * The values are computed based in the settings of \c gpgcheck, \c repo_gpgcheck
+       * end \c pkg_gpgcheck in \c zypp.conf. Explicitly setting these values in the
+       * repositories \a .repo file will overwrite the defaults from \c zypp.conf for this
+       * repo.
        *
-       * The default is \c true but may be overwritten by \c zypp.conf or a \ref .repo file.
+       * If \c gpgcheck is \c on (the default) we will check the signature of repo metadata
+       * (packages are secured via checksum inside the metadata). Using unsigned repos
+       * needs to be confirmed.
+       * Packages from signed repos are accepted if their checksum matches the checksum
+       * stated in the repo metadata.
+       * Packages from unsigned repos need a valid gpg signature, using unsigned packages
+       * needs to be confirmed.
+       *
+       * The above default behavior can be tuned by explicitly setting \c repo_gpgcheck
+       * and/or \c pkg_gpgcheck:
+       *
+       *   \c repo_gpgcheck = \c on same as the default.
+       *
+       *   \c repo_gpgcheck = \c off will silently accept unsigned repos. It will NOT turn of
+       *   signature checking on the whole, nevertheless it's not a secure setting.
+       *
+       *   \c pkg_gpgcheck = \c on will enforce the package signature checking and the need
+       *   to confirm unsigned packages for all repos (signed and unsigned).
+       *
+       *   \c pkg_gpgcheck = \c off will silently accept unsigned packages. It will NOT turn of
+       *   signature checking on the whole, nevertheless it's not a secure setting.
+       *
+       * If \c gpgCheck is \c off (not recommneded), no checks are performed. You can still
+       * enable them individually by setting \c repo_gpgcheck and/or \c pkg_gpgcheck to \c on.
+       *
+       * \code
+       *  R: check repo signature is mandatory, confirm unsigned repos
+       *  r: check repo signature, unsigned repos are ok but enforce p
+       *   : do not check repo signatures
+       *
+       *  P: check package signature always, confirm unsigned packages
+       *  p: like P for unsigned repos, accepted by checksum for signed repos
+       *  b: like p but accept unsigned packages
+       *   : do not check package signatures
+       *                    pkg_
+       * gpgcheck 1|     *       0       1
+       * ------------------------------------
+       * repo_   *1|     R/p     R/b     R/P
+       *          0|     r/p     r/b     r/P
+       *
+       *                    pkg_
+       * gpgcheck 0|     *       0       1
+       * ------------------------------------
+       * repo_   *0|                       P
+       *          1|     R       R       R/P
+       * \endcode
        */
+      //@{
+      /** Whether default signature checking should be performed. */
       bool gpgCheck() const;
       /** Set the value for \ref gpgCheck (or \c indeterminate to use the default). */
       void setGpgCheck( TriBool value_r );
       /** \overload \deprecated legacy and for squid */
       void setGpgCheck( bool value_r );
 
-      /** Whether the signature of repo metadata should be checked for this repo.
-       * The default is defined by \ref gpgCheck but may be overwritten by \c zypp.conf or a \ref .repo file.
-       */
+      /** Whether the signature of repo metadata should be checked for this repo. */
       bool repoGpgCheck() const;
+      /** Mandatory check (\ref repoGpgCheck is \c on) must ask to confirm using unsigned repos. */
+      bool repoGpgCheckIsMandatory() const;
       /** Set the value for \ref repoGpgCheck (or \c indeterminate to use the default). */
       void setRepoGpgCheck( TriBool value_r );
 
-      /** Whether the signature of rpm packages should be checked for this repo.
-       * The default is defined by \ref gpgCheck but may be overwritten by \c zypp.conf or a \ref .repo file.
-       */
+      /** Whether the signature of rpm packages should be checked for this repo. */
       bool pkgGpgCheck() const;
+      /** Mandatory check (\ref pkgGpgCheck is not \c off) must ask to confirm using unsigned packages. */
+      bool pkgGpgCheckIsMandatory() const;
       /** Set the value for \ref pkgGpgCheck (or \c indeterminate to use the default). */
       void setPkgGpgCheck( TriBool value_r );
 
@@ -290,6 +361,25 @@ namespace zypp
       /** Set the value for \ref validRepoSignature (or \c indeterminate if unsigned). */
       void setValidRepoSignature( TriBool value_r );
 
+      /** Some predefined settings */
+      enum class GpgCheck {
+	indeterminate,		//< not specified
+	On,			//< 1** --gpgcheck
+	Strict,			//< 111 --gpgcheck-strict
+	AllowUnsigned,		//< 100 --gpgcheck-allow-unsigned
+	AllowUnsignedRepo,	//< 10* --gpgcheck-allow-unsigned-repo
+	AllowUnsignedPackage,	//< 1*0 --gpgcheck-allow-unsigned-package
+	Default,		//< *** --default-gpgcheck
+	Off,			//< 0** --no-gpgcheck
+      };
+
+      /** Adjust *GpgCheck settings according to \a mode_r.
+       * \c GpgCheck::indeterminate will leave the settings as they are.
+       * \return whether setting were changed
+       */
+      bool setGpgCheck( GpgCheck mode_r );
+      //@}
+
 
       /** Whether gpgkey URLs are defined */
       bool gpgKeyUrlsEmpty() const;
@@ -310,6 +400,8 @@ namespace zypp
       /** (leagcy API) Set the gpgkey URL defined for this repo */
       void setGpgKeyUrl( const Url &gpgkey );
 
+      /** downloads all configured gpg keys into the defined directory */
+      Pathname provideKey(const std::string &keyID_r, const Pathname &targetDirectory_r ) const;
 
       /**
        * \short Whether packages downloaded from this repository will be kept in local cache
@@ -386,27 +478,45 @@ namespace zypp
       { return hasContentAny( container_r.begin(), container_r.end() ); }
 
     public:
-      /** \name Repository license
-      */
+      /** \name Repository/Product license
+       * In case a repository provides multiple license tarballs in repomd.xml
+       * \code
+       *   <data type="license">...</data>
+       *   <data type="license-sles">...</data>
+       *   <data type="license-sled">...</data>
+       * \endcode
+       * you can address the individual licenses by passing their name
+       * (e.g. \c "sles" to access the \c type="license-sles").
+       * No on an empty name will refer to \c type="license".
+       */
       //@{
       /** Whether there is a license associated with the repo. */
       bool hasLicense() const;
+      /** \overload taking a (product)name */
+      bool hasLicense( const std::string & name_r ) const;
 
       /** Whether the repo license has to be accepted, e.g. there is no
        * no acceptance needed for openSUSE.
        */
       bool needToAcceptLicense() const;
+      /** \overload taking a (product)name */
+      bool needToAcceptLicense( const std::string & name_r ) const;
 
       /** Return the best license for the current (or a specified) locale. */
       std::string getLicense( const Locale & lang_r = Locale() ) const;
+      /** \overload not const LEGACY API */
       std::string getLicense( const Locale & lang_r = Locale() ); // LEGACY API
+      /** \overload taking a (product)name */
+      std::string getLicense( const std::string & name_r, const Locale & lang_r = Locale() ) const;
 
       /** Return the locales the license is available for.
        * \ref Locale::noCode is included in case of \c license.txt which does
        * not specify a specific locale.
        */
       LocaleSet getLicenseLocales() const;
-      //@}
+      /** \overload taking a (product)name */
+      LocaleSet getLicenseLocales( const std::string & name_r ) const;
+     //@}
 
     public:
       /**
@@ -452,6 +562,8 @@ namespace zypp
   /** \relates RepoInfo Stream output */
   std::ostream & operator<<( std::ostream & str, const RepoInfo & obj );
 
+  /** \relates RepoInfo::GpgCheck Stream output */
+  std::ostream & operator<<( std::ostream & str, const RepoInfo::GpgCheck & obj );
 
   /////////////////////////////////////////////////////////////////
 } // namespace zypp
