@@ -20,6 +20,7 @@
 
 #include <solv/solvversion.h>
 
+#include "zypp/base/String.h"
 #include "zypp/base/InputStream.h"
 #include "zypp/base/LogTools.h"
 #include "zypp/base/Gettext.h"
@@ -38,6 +39,7 @@
 #include "zypp/MediaSetAccess.h"
 #include "zypp/ExternalProgram.h"
 #include "zypp/ManagedFile.h"
+#include "zypp/KeyManager.h"
 
 #include "zypp/parser/RepoFileReader.h"
 #include "zypp/parser/ServiceFileReader.h"
@@ -63,6 +65,19 @@ using namespace zypp::repo;
 ///////////////////////////////////////////////////////////////////
 namespace zypp
 {
+
+  ///////////////////////////////////////////////////////////////////
+  namespace env
+  {
+    /** To trigger appdata refresh unconditionally */
+    inline bool ZYPP_PLUGIN_APPDATA_FORCE_COLLECT()
+    {
+      const char * env = getenv("ZYPP_PLUGIN_APPDATA_FORCE_COLLECT");
+      return( env && str::strToBool( env, true ) );
+    }
+  } // namespace env
+  ///////////////////////////////////////////////////////////////////
+
   ///////////////////////////////////////////////////////////////////
   namespace
   {
@@ -296,7 +311,7 @@ namespace zypp
       bool nonroot( geteuid() != 0 );
       if ( nonroot && ! PathInfo(dir).userMayRX() )
       {
-	JobReport::warning( str::FormatNAC(_("Cannot read repo directory '%1%': Permission denied")) % dir );
+	JobReport::warning( str::Format(_("Cannot read repo directory '%1%': Permission denied")) % dir );
       }
       else
       {
@@ -314,7 +329,7 @@ namespace zypp
 	  {
 	    if ( nonroot && ! PathInfo(*it).userMayR() )
 	    {
-	      JobReport::warning( str::FormatNAC(_("Cannot read repo file '%1%': Permission denied")) % *it );
+	      JobReport::warning( str::Format(_("Cannot read repo file '%1%': Permission denied")) % *it );
 	    }
 	    else
 	    {
@@ -367,6 +382,15 @@ namespace zypp
 
     ////////////////////////////////////////////////////////////////////////////
 
+    ///////////////////////////////////////////////////////////////////
+    namespace
+    {
+      /** Whether repo is not under RM control and provides it's own methadata paths. */
+      inline bool isTmpRepo( const RepoInfo & info_r )
+      { return( info_r.filepath().empty() && info_r.usesAutoMethadataPaths() ); }
+    } // namespace
+    ///////////////////////////////////////////////////////////////////
+
     /**
      * \short Calculates the raw cache path for a repository, this is usually
      * /var/cache/zypp/alias
@@ -374,7 +398,7 @@ namespace zypp
     inline Pathname rawcache_path_for_repoinfo( const RepoManagerOptions &opt, const RepoInfo &info )
     {
       assert_alias(info);
-      return opt.repoRawCachePath / info.escaped_alias();
+      return isTmpRepo( info ) ? info.metadataPath() : opt.repoRawCachePath / info.escaped_alias();
     }
 
     /**
@@ -386,10 +410,7 @@ namespace zypp
      * for example /var/cache/zypp/alias/addondir
      */
     inline Pathname rawproductdata_path_for_repoinfo( const RepoManagerOptions &opt, const RepoInfo &info )
-    {
-      assert_alias(info);
-      return opt.repoRawCachePath / info.escaped_alias() / info.path();
-    }
+    { return rawcache_path_for_repoinfo( opt, info ) / info.path(); }
 
     /**
      * \short Calculates the packages cache path for a repository
@@ -397,16 +418,16 @@ namespace zypp
     inline Pathname packagescache_path_for_repoinfo( const RepoManagerOptions &opt, const RepoInfo &info )
     {
       assert_alias(info);
-      return opt.repoPackagesCachePath / info.escaped_alias();
+      return isTmpRepo( info ) ? info.packagesPath() : opt.repoPackagesCachePath / info.escaped_alias();
     }
 
     /**
      * \short Calculates the solv cache path for a repository
      */
-    inline Pathname solv_path_for_repoinfo( const RepoManagerOptions &opt, const RepoInfo &info)
+    inline Pathname solv_path_for_repoinfo( const RepoManagerOptions &opt, const RepoInfo &info )
     {
       assert_alias(info);
-      return opt.repoSolvCachePath / info.escaped_alias();
+      return isTmpRepo( info ) ? info.metadataPath().dirname() / "%SLV%" : opt.repoSolvCachePath / info.escaped_alias();
     }
 
     ////////////////////////////////////////////////////////////////////////////
@@ -437,13 +458,7 @@ namespace zypp
 
   std::list<RepoInfo> readRepoFile( const Url & repo_file )
   {
-    // no interface to download a specific file, using workaround:
-    //! \todo add MediaManager::provideFile(Url file_url) to easily access any file URLs? (no need for media access id or media_nr)
-    Url url(repo_file);
-    Pathname path(url.getPathName());
-    url.setPathName ("/");
-    MediaSetAccess access(url);
-    Pathname local = access.provideFile(path);
+    ManagedFile local = MediaSetAccess::provideFileFromUrl(repo_file);
 
     DBG << "reading repo file " << repo_file << ", local path: " << local << endl;
 
@@ -517,7 +532,8 @@ namespace zypp
     ~Impl()
     {
       // trigger appdata refresh if some repos change
-      if ( _reposDirty && geteuid() == 0 && ( _options.rootDir.empty() || _options.rootDir == "/" ) )
+      if ( ( _reposDirty || env::ZYPP_PLUGIN_APPDATA_FORCE_COLLECT() )
+	&& geteuid() == 0 && ( _options.rootDir.empty() || _options.rootDir == "/" ) )
       {
 	try {
 	  std::list<Pathname> entries;
@@ -776,7 +792,7 @@ namespace zypp
       }
     }
 
-    repo::PluginServices(_options.pluginsPath/"services", ServiceCollector(_services));
+   repo::PluginServices(_options.pluginsPath/"services", ServiceCollector(_services));
   }
 
   ///////////////////////////////////////////////////////////////////
@@ -848,7 +864,7 @@ namespace zypp
 	  // translators: Cleanup a repository previously owned by a meanwhile unknown (deleted) service.
 	  //   %1% = service name
 	  //   %2% = repository name
-	  JobReport::warning( str::FormatNAC(_("Unknown service '%1%': Removing orphaned service repository '%2%'"))
+	  JobReport::warning( str::Format(_("Unknown service '%1%': Removing orphaned service repository '%2%'"))
 			      % repoInfo.service()
 			      % repoInfo.alias() );
 	  try {
@@ -891,7 +907,7 @@ namespace zypp
     switch ( repokind.toEnum() )
     {
       case RepoType::RPMMD_e :
-        status = RepoStatus( productdatapath/"repodata/repomd.xml");
+        status = RepoStatus( productdatapath/"repodata/repomd.xml") && RepoStatus( mediarootpath/"media.1/media" );
 	break;
 
       case RepoType::YAST2_e :
@@ -954,33 +970,37 @@ namespace zypp
     assert_alias(info);
     try
     {
-      MIL << "Going to try to check whether refresh is needed for " << url << endl;
+      MIL << "Going to try to check whether refresh is needed for " << url << " (" << info.type() << ")" << endl;
 
       // first check old (cached) metadata
       Pathname mediarootpath = rawcache_path_for_repoinfo( _options, info );
       filesystem::assert_dir( mediarootpath );
       RepoStatus oldstatus = metadataStatus( info );
-
       if ( oldstatus.empty() )
       {
         MIL << "No cached metadata, going to refresh" << endl;
         return REFRESH_NEEDED;
       }
 
+      if ( url.schemeIsVolatile() )
       {
-        if ( url.schemeIsVolatile() )
-	{
-	  MIL << "never refresh CD/DVD" << endl;
-          return REPO_UP_TO_DATE;
-	}
-	if ( url.schemeIsLocal() )
-	{
-	  policy = RefreshIfNeededIgnoreDelay;
-	}
+	MIL << "Never refresh CD/DVD" << endl;
+	return REPO_UP_TO_DATE;
+      }
+
+      if ( policy == RefreshForced )
+      {
+	MIL << "Forced refresh!" << endl;
+	return REFRESH_NEEDED;
+      }
+
+      if ( url.schemeIsLocal() )
+      {
+	policy = RefreshIfNeededIgnoreDelay;
       }
 
       // now we've got the old (cached) status, we can decide repo.refresh.delay
-      if (policy != RefreshForced && policy != RefreshIfNeededIgnoreDelay)
+      if ( policy != RefreshIfNeededIgnoreDelay )
       {
         // difference in seconds
         double diff = difftime(
@@ -1042,27 +1062,17 @@ namespace zypp
       }
 
       // check status
-      bool refresh = false;
       if ( oldstatus == newstatus )
       {
 	MIL << "repo has not changed" << endl;
-	if ( policy == RefreshForced )
-	{
-	  MIL << "refresh set to forced" << endl;
-	  refresh = true;
-	}
+	touchIndexFile( info );
+	return REPO_UP_TO_DATE;
       }
-      else
+      else // includes newstatus.empty() if e.g. repo format changed
       {
 	MIL << "repo has changed, going to refresh" << endl;
-	refresh = true;
+	return REFRESH_NEEDED;
       }
-
-      if (!refresh)
-	touchIndexFile(info);
-
-      return refresh ? REFRESH_NEEDED : REPO_UP_TO_DATE;
-
     }
     catch ( const Exception &e )
     {
@@ -1087,7 +1097,6 @@ namespace zypp
 
     // Suppress (interactive) media::MediaChangeReport if we in have multiple basurls (>1)
     media::ScopedDisableMediaChangeReport guard( info.baseUrlsSize() > 1 );
-
     // try urls one by one
     for ( RepoInfo::urls_const_iterator it = info.baseUrlsBegin(); it != info.baseUrlsEnd(); ++it )
     {
@@ -1102,16 +1111,15 @@ namespace zypp
 
         MIL << "Going to refresh metadata from " << url << endl;
 
+	// bsc#1048315: Always re-probe in case of repo format change.
+	// TODO: Would be sufficient to verify the type and re-probe
+	// if verification failed (or type is RepoType::NONE)
         repo::RepoType repokind = info.type();
-
-        // if the type is unknown, try probing.
-	if ( repokind == RepoType::NONE )
 	{
-	  // unknown, probe it
-	  repokind = probe( *it, info.path() );
-
-	  if (repokind.toEnum() != RepoType::NONE_e)
+	  repo::RepoType probed = probe( *it, info.path() );
+	  if ( repokind != probed )
 	  {
+	    repokind = probed;
 	    // Adjust the probed type in RepoInfo
 	    info.setProbedType( repokind ); // lazy init!
 	    //save probed type only for repos in system
@@ -1188,7 +1196,8 @@ namespace zypp
         // ok we have the metadata, now exchange
         // the contents
 	filesystem::exchange( tmpdir.path(), mediarootpath );
-	reposManip();	// remember to trigger appdata refresh
+	if ( ! isTmpRepo( info ) )
+	  reposManip();	// remember to trigger appdata refresh
 
         // we are done.
         return;
@@ -1341,10 +1350,11 @@ namespace zypp
         cmd.push_back( "-o" );
         cmd.push_back( solvfile.asString() );
 	cmd.push_back( "-X" );	// autogenerate pattern from pattern-package
+        // bsc#1104415: no more application support // cmd.push_back( "-A" );	// autogenerate application pseudo packages
 
         if ( repokind == RepoType::RPMPLAINDIR )
         {
-          forPlainDirs.reset( new MediaMounter( *info.baseUrlsBegin() ) );
+          forPlainDirs.reset( new MediaMounter( info.url() ) );
           // recusive for plaindir as 2nd arg!
           cmd.push_back( "-R" );
           // FIXME this does only work form dir: URLs
@@ -1620,16 +1630,13 @@ namespace zypp
     if ( _options.probe )
     {
       DBG << "unknown repository type, probing" << endl;
+      assert_urls(tosave);
 
-      RepoType probedtype;
-      probedtype = probe( *tosave.baseUrlsBegin(), info.path() );
-      if ( tosave.baseUrlsSize() > 0 )
-      {
-        if ( probedtype == RepoType::NONE )
-          ZYPP_THROW(RepoUnknownTypeException(info));
-        else
-          tosave.setType(probedtype);
-      }
+      RepoType probedtype( probe( tosave.url(), info.path() ) );
+      if ( probedtype == RepoType::NONE )
+	ZYPP_THROW(RepoUnknownTypeException(info));
+      else
+	tosave.setType(probedtype);
     }
 
     progress.set(50);
@@ -1651,14 +1658,15 @@ namespace zypp
 
     tosave.dumpAsIniOn(file);
     tosave.setFilepath(repofile);
-    tosave.setMetadataPath( metadataPath( tosave ) );
-    tosave.setPackagesPath( packagesPath( tosave ) );
+    tosave.setMetadataPath( rawcache_path_for_repoinfo( _options, tosave ) );
+    tosave.setPackagesPath( packagescache_path_for_repoinfo( _options, tosave ) );
     {
-      // We chould fix the API as we must injet those paths
+      // We should fix the API as we must inject those paths
       // into the repoinfo in order to keep it usable.
       RepoInfo & oinfo( const_cast<RepoInfo &>(info) );
-      oinfo.setMetadataPath( metadataPath( tosave ) );
-      oinfo.setPackagesPath( packagesPath( tosave ) );
+      oinfo.setFilepath(repofile);
+      oinfo.setMetadataPath( rawcache_path_for_repoinfo( _options, tosave ) );
+      oinfo.setPackagesPath( packagescache_path_for_repoinfo( _options, tosave ) );
     }
     reposManip().insert(tosave);
 
@@ -1719,8 +1727,10 @@ namespace zypp
           ++it )
     {
       MIL << "Saving " << (*it).alias() << endl;
-      it->setFilepath(repofile.asString());
       it->dumpAsIniOn(file);
+      it->setFilepath(repofile);
+      it->setMetadataPath( rawcache_path_for_repoinfo( _options, *it ) );
+      it->setPackagesPath( packagescache_path_for_repoinfo( _options, *it ) );
       reposManip().insert(*it);
 
       HistoryLog(_options.rootDir).addRepository(*it);
@@ -1875,6 +1885,16 @@ namespace zypp
       }
 
       newinfo.setFilepath(toedit.filepath());
+      newinfo.setMetadataPath( rawcache_path_for_repoinfo( _options, newinfo ) );
+      newinfo.setPackagesPath( packagescache_path_for_repoinfo( _options, newinfo ) );
+      {
+	// We should fix the API as we must inject those paths
+	// into the repoinfo in order to keep it usable.
+	RepoInfo & oinfo( const_cast<RepoInfo &>(newinfo_r) );
+	oinfo.setFilepath(toedit.filepath());
+	oinfo.setMetadataPath( rawcache_path_for_repoinfo( _options, newinfo ) );
+	oinfo.setPackagesPath( packagescache_path_for_repoinfo( _options, newinfo ) );
+      }
       reposManip().erase(toedit);
       reposManip().insert(newinfo);
       // check for credentials in Urls
@@ -2076,7 +2096,13 @@ namespace zypp
     // and in zypper.
     std::pair<DefaultIntegral<bool,false>, repo::ServicePluginInformalException> uglyHack;
     try {
-      ServiceRepos( service, bind( &RepoCollector::collect, &collector, _1 ) );
+      // FIXME bsc#1080693: Shortcoming of (plugin)services (and repos as well) is that they
+      // are not aware of the RepoManagers rootDir. The service url, as created in known_services,
+      // contains the full path to the script. The script however has to be executed chrooted.
+      // Repos would need to know the RepoMangers rootDir to use the correct vars.d to replace
+      // repos variables. Until RepoInfoBase is aware if the rootDir, we need to explicitly pass it
+      // to ServiceRepos.
+      ServiceRepos( _options.rootDir, service, bind( &RepoCollector::collect, &collector, _1 ) );
     }
     catch ( const repo::ServicePluginInformalException & e )
     {
