@@ -309,7 +309,7 @@ namespace zypp
         : _parsedZyppConf         	( override_r )
         , cfg_arch                	( defaultSystemArchitecture() )
         , cfg_textLocale          	( defaultTextLocale() )
-        , updateMessagesNotify		( "single | /usr/lib/zypp/notify-message -p %p" )
+        , updateMessagesNotify		( "" )
         , repo_add_probe          	( false )
         , repo_refresh_delay      	( 10 )
         , repoLabelIsAlias              ( false )
@@ -407,6 +407,10 @@ namespace zypp
                 {
                   cfg_known_services_path = Pathname(value);
                 }
+                else if ( entry == "varsdir" )
+                {
+                  cfg_vars_path = Pathname(value);
+                }
                 else if ( entry == "repo.add.probe" )
                 {
                   repo_add_probe = str::strToBool( value, repo_add_probe );
@@ -472,15 +476,15 @@ namespace zypp
                 }
                 else if ( entry == "gpgcheck" )
 		{
-		  gpgCheck.set( str::strToBool( value, gpgCheck ) );
+		  gpgCheck.restoreToDefault( str::strToBool( value, gpgCheck ) );
 		}
 		else if ( entry == "repo_gpgcheck" )
 		{
-		  repoGpgCheck.set( str::strToTriBool( value ) );
+		  repoGpgCheck.restoreToDefault( str::strToTriBool( value ) );
 		}
 		else if ( entry == "pkg_gpgcheck" )
 		{
-		  pkgGpgCheck.set( str::strToTriBool( value ) );
+		  pkgGpgCheck.restoreToDefault( str::strToTriBool( value ) );
 		}
                 else if ( entry == "vendordir" )
                 {
@@ -623,6 +627,8 @@ namespace zypp
     Pathname cfg_config_path;
     Pathname cfg_known_repos_path;
     Pathname cfg_known_services_path;
+    Pathname cfg_vars_path;
+    Pathname cfg_repo_mgr_root_path;
 
     Pathname cfg_vendor_path;
     Pathname cfg_multiversion_path;
@@ -651,9 +657,9 @@ namespace zypp
 
     Option<DownloadMode> commit_downloadMode;
 
-    Option<bool>	gpgCheck;
-    Option<TriBool>	repoGpgCheck;
-    Option<TriBool>	pkgGpgCheck;
+    DefaultOption<bool>		gpgCheck;
+    DefaultOption<TriBool>	repoGpgCheck;
+    DefaultOption<TriBool>	pkgGpgCheck;
 
     Option<bool>	solver_onlyRequires;
     Option<bool>	solver_allowVendorChange;
@@ -813,6 +819,16 @@ namespace zypp
   Pathname ZConfig::systemRoot() const
   { return _autodetectSystemRoot(); }
 
+
+  Pathname ZConfig::repoManagerRoot() const
+  {
+    return ( _pimpl->cfg_repo_mgr_root_path.empty()
+             ? systemRoot() : _pimpl->cfg_repo_mgr_root_path );
+  }
+
+  void ZConfig::setRepoManagerRoot(const zypp::filesystem::Pathname &root)
+  { _pimpl->cfg_repo_mgr_root_path = root; }
+
   ///////////////////////////////////////////////////////////////////
   //
   // system architecture
@@ -893,7 +909,18 @@ namespace zypp
   Pathname ZConfig::repoCachePath() const
   {
     return ( _pimpl->cfg_cache_path.empty()
-        ? Pathname("/var/cache/zypp") : _pimpl->cfg_cache_path );
+             ? Pathname("/var/cache/zypp") : _pimpl->cfg_cache_path );
+  }
+
+  Pathname ZConfig::pubkeyCachePath() const
+  {
+    return ( _pimpl->cfg_cache_path.empty()
+             ? Pathname("/var/cache/zypp/pubkeys") : _pimpl->cfg_cache_path/"pubkeys" );
+  }
+
+  void ZConfig::setRepoCachePath(const zypp::filesystem::Pathname &path_r)
+  {
+    _pimpl->cfg_cache_path = path_r;
   }
 
   Pathname ZConfig::repoMetadataPath() const
@@ -902,18 +929,33 @@ namespace zypp
         ? (repoCachePath()/"raw") : _pimpl->cfg_metadata_path );
   }
 
+  void ZConfig::setRepoMetadataPath(const zypp::filesystem::Pathname &path_r)
+  {
+    _pimpl->cfg_metadata_path = path_r;
+  }
+
   Pathname ZConfig::repoSolvfilesPath() const
   {
     return ( _pimpl->cfg_solvfiles_path.empty()
         ? (repoCachePath()/"solv") : _pimpl->cfg_solvfiles_path );
   }
 
+  void ZConfig::setRepoSolvfilesPath(const zypp::filesystem::Pathname &path_r)
+  {
+    _pimpl->cfg_solvfiles_path = path_r;
+  }
+
   Pathname ZConfig::repoPackagesPath() const
   {
     return ( _pimpl->cfg_packages_path.empty()
         ? (repoCachePath()/"packages") : _pimpl->cfg_packages_path );
   }
 
+  void ZConfig::setRepoPackagesPath(const zypp::filesystem::Pathname &path_r)
+  {
+    _pimpl->cfg_packages_path = path_r;
+  }
+
   ///////////////////////////////////////////////////////////////////
 
   Pathname ZConfig::configPath() const
@@ -934,6 +976,12 @@ namespace zypp
         ? (configPath()/"services.d") : _pimpl->cfg_known_services_path );
   }
 
+  Pathname ZConfig::varsPath() const
+  {
+    return ( _pimpl->cfg_vars_path.empty()
+        ? (configPath()/"vars.d") : _pimpl->cfg_vars_path );
+  }
+
   Pathname ZConfig::vendorPath() const
   {
     return ( _pimpl->cfg_vendor_path.empty()
@@ -1000,14 +1048,19 @@ namespace zypp
   DownloadMode ZConfig::commit_downloadMode() const
   { return _pimpl->commit_downloadMode; }
 
-  bool ZConfig::gpgCheck() const
-  { return _pimpl->gpgCheck; }
 
-  TriBool ZConfig::repoGpgCheck() const
-  { return _pimpl->repoGpgCheck; }
+  bool ZConfig::gpgCheck() const			{ return _pimpl->gpgCheck; }
+  TriBool ZConfig::repoGpgCheck() const			{ return _pimpl->repoGpgCheck; }
+  TriBool ZConfig::pkgGpgCheck() const			{ return _pimpl->pkgGpgCheck; }
+
+  void ZConfig::setGpgCheck( bool val_r )		{ _pimpl->gpgCheck.set( val_r ); }
+  void ZConfig::setRepoGpgCheck( TriBool val_r )	{ _pimpl->repoGpgCheck.set( val_r ); }
+  void ZConfig::setPkgGpgCheck( TriBool val_r )		{ _pimpl->pkgGpgCheck.set( val_r ); }
+
+  void ZConfig::resetGpgCheck()				{ _pimpl->gpgCheck.restoreToDefault(); }
+  void ZConfig::resetRepoGpgCheck()			{ _pimpl->repoGpgCheck.restoreToDefault(); }
+  void ZConfig::resetPkgGpgCheck()			{ _pimpl->pkgGpgCheck.restoreToDefault(); }
 
-  TriBool ZConfig::pkgGpgCheck() const
-  { return _pimpl->pkgGpgCheck; }
 
   bool ZConfig::solver_onlyRequires() const
   { return _pimpl->solver_onlyRequires; }
@@ -1109,7 +1162,7 @@ namespace zypp
   ///////////////////////////////////////////////////////////////////
 
   std::string ZConfig::distroverpkg() const
-  { return "redhat-release"; }
+  { return "system-release"; }
 
   ///////////////////////////////////////////////////////////////////
 
