@@ -73,6 +73,16 @@ namespace zypp
        */
       Pathname systemRoot() const;
 
+      /** The RepoManager root directory.
+       *  Returns the same as \sa systemRoot() if not explicitely set.
+       */
+      Pathname repoManagerRoot() const;
+
+      /** Sets the RepoManager root directory.
+       *  \sa repoManagerRoot()
+       */
+      void setRepoManagerRoot ( const Pathname &root );
+
     public:
 
       /** The autodetected system architecture. */
@@ -134,24 +144,50 @@ namespace zypp
        */
       Pathname repoCachePath() const;
 
+      /**
+       * Set a new \a path as the default repo cache path
+       */
+      void setRepoCachePath ( const Pathname &path_r );
+
+      /**
+       * Path where the pubkey caches
+       */
+      Pathname pubkeyCachePath() const;
+
      /**
        * Path where the repo metadata is downloaded and kept (repoCachePath()/raw).
         * \ingroup g_ZC_REPOCACHE
       */
       Pathname repoMetadataPath() const;
 
+
+      /**
+       * Set a new \a path as the default repo cache path
+       */
+      void setRepoMetadataPath ( const Pathname &path_r );
+
      /**
        * Path where the repo solv files are created and kept (repoCachePath()/solv).
         * \ingroup g_ZC_REPOCACHE
       */
       Pathname repoSolvfilesPath() const;
 
+      /**
+       * Set a new \a path as the default repo cache path
+       */
+      void setRepoSolvfilesPath ( const Pathname &path_r );
+
       /**
        * Path where the repo packages are downloaded and kept (repoCachePath()/packages).
         * \ingroup g_ZC_REPOCACHE
       */
       Pathname repoPackagesPath() const;
 
+      /**
+       * Set a new \a path as the default repo cache path
+       */
+      void setRepoPackagesPath ( const Pathname &path_r );
+
       /**
        * Path where the configfiles are kept (/etc/zypp).
        * \ingroup g_ZC_CONFIGFILES
@@ -170,6 +206,13 @@ namespace zypp
        */
       Pathname knownServicesPath() const;
 
+      /**
+       * Path containing custom repo variable definitions (configPath()/vars.d).
+       * \see \ref zypp::repo::RepoVarExpand Repo variable expander
+       * \ingroup g_ZC_CONFIGFILES
+       */
+      Pathname varsPath() const;
+
       /**
        * Whether repository urls should be probed.
        / config option
@@ -286,10 +329,18 @@ namespace zypp
        * repositories \a .repo file will overwrite the defaults here.
        */
       //@{
-      bool gpgCheck() const;		///< Turn signature checking on/off (on)
-      TriBool repoGpgCheck() const;	///< Check repo matadata signatures (indeterminate - according to gpgcheck)
-      TriBool pkgGpgCheck() const;	///< Check rpm package signatures (indeterminate - according to gpgcheck)
-      //@}
+      bool gpgCheck() const;			///< Turn signature checking on/off (on)
+      TriBool repoGpgCheck() const;		///< Check repo matadata signatures (indeterminate - according to gpgcheck)
+      TriBool pkgGpgCheck() const;		///< Check rpm package signatures (indeterminate - according to gpgcheck)
+
+      void setGpgCheck( bool val_r );		///< Change the value
+      void setRepoGpgCheck( TriBool val_r );	///< Change the value
+      void setPkgGpgCheck( TriBool val_r );	///< Change the value
+
+      void resetGpgCheck();			///< Reset to the zconfig default
+      void resetRepoGpgCheck();			///< Reset to the zconfig default
+      void resetPkgGpgCheck();			///< Reset to the zconfig default
+     //@}
       //
       /**
        * Directory for equivalent vendor definitions  (configPath()/vendors.d)
@@ -452,10 +503,10 @@ namespace zypp
       /** Package telling the "product version" on systems not using /etc/product.d/baseproduct.
        *
        * On RHEL, Fedora and others the "product version" is determined by the first package
-       * providing 'redhat-release'. This value is not hardcoded in YUM and can be configured
+       * providing 'system-release'. This value is not hardcoded in YUM and can be configured
        * with the $distroverpkg variable.
        *
-       * Defaults to 'redhat-release'.
+       * Defaults to 'system-release'.
        */
       std::string distroverpkg() const;
 
@@ -471,7 +522,7 @@ namespace zypp
       class Impl;
       /** Dtor */
       ~ZConfig();
-    private:
+  private:
       friend class Impl;
       /** Default ctor. */
       ZConfig();
