@@ -421,12 +421,12 @@ namespace zypp
     //TODO: make configurable
     list<string> filesystems;
 
-    // if DVD, try UDF filesystem before iso9660
+    filesystems.push_back("iso9660");
+
+    // if DVD, try UDF filesystem after iso9660
     if ( _url.getScheme() == "dvd" )
       filesystems.push_back("udf");
 
-    filesystems.push_back("iso9660");
-
     // try all devices in sequence
     int count = 0;
     std::string mountpoint( attachPoint().asString() );
@@ -720,9 +720,9 @@ namespace zypp
   //
   //  DESCRIPTION : Asserted that media is attached.
   //
-  void MediaCD::getFile( const Pathname & filename ) const
+  void MediaCD::getFile(const Pathname & filename , const ByteCount &expectedFileSize_r) const
   {
-    MediaHandler::getFile( filename );
+    MediaHandler::getFile( filename, expectedFileSize_r );
   }
 
   ///////////////////////////////////////////////////////////////////
