@@ -151,25 +151,57 @@ namespace
 }
 
 namespace zypp {
+
+  ///////////////////////////////////////////////////////////////////
+  namespace env
+  {
+    namespace
+    {
+      inline int getZYPP_MEDIA_CURL_IPRESOLVE()
+      {
+	int ret = 0;
+	if ( const char * envp = getenv( "ZYPP_MEDIA_CURL_IPRESOLVE" ) )
+	{
+	  WAR << "env set: $ZYPP_MEDIA_CURL_IPRESOLVE='" << envp << "'" << endl;
+	  if (      strcmp( envp, "4" ) == 0 )	ret = 4;
+	  else if ( strcmp( envp, "6" ) == 0 )	ret = 6;
+	}
+	return ret;
+      }
+    }
+
+    inline int ZYPP_MEDIA_CURL_IPRESOLVE()
+    {
+      static int _v = getZYPP_MEDIA_CURL_IPRESOLVE();
+      return _v;
+    }
+  } // namespace env
+  ///////////////////////////////////////////////////////////////////
+
   namespace media {
 
   namespace {
     struct ProgressData
     {
       ProgressData( CURL *_curl, time_t _timeout = 0, const Url & _url = Url(),
+                    ByteCount expectedFileSize_r = 0,
 		    callback::SendReport<DownloadProgressReport> *_report = nullptr )
         : curl( _curl )
 	, url( _url )
 	, timeout( _timeout )
         , reached( false )
+        , fileSizeExceeded ( false )
         , report( _report )
+        , _expectedFileSize( expectedFileSize_r )
       {}
 
       CURL	*curl;
       Url	url;
       time_t	timeout;
       bool	reached;
+      bool      fileSizeExceeded;
       callback::SendReport<DownloadProgressReport> *report;
+      ByteCount _expectedFileSize;
 
       time_t _timeStart	= 0;	///< Start total stats
       time_t _timeLast	= 0;	///< Start last period(~1sec)
@@ -213,6 +245,9 @@ namespace zypp {
 	if ( timeout )
 	  reached = ( (now - _timeRcv) > timeout );
 
+        // check if the downloaded data is already bigger than what we expected
+  	fileSizeExceeded = _expectedFileSize > 0 && _expectedFileSize < static_cast<ByteCount::SizeType>(_dnlNow);
+
 	// percentage:
 	if ( _dnlTotal )
 	  _dnlPercent = int(_dnlNow * 100 / _dnlTotal);
@@ -233,6 +268,8 @@ namespace zypp {
 
       int reportProgress() const
       {
+        if ( fileSizeExceeded )
+          return 1;
 	if ( reached )
 	  return 1;	// no-data timeout
 	if ( report && !(*report)->progress( _dnlPercent, url, _drateTotal, _drateLast ) )
@@ -672,6 +709,16 @@ void MediaCurl::setupEasy()
       fillSettingsSystemProxy(_url, _settings);
   }
 
+  /** Force IPv4/v6 */
+  if ( env::ZYPP_MEDIA_CURL_IPRESOLVE() )
+  {
+    switch ( env::ZYPP_MEDIA_CURL_IPRESOLVE() )
+    {
+      case 4: SET_OPTION(CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4); break;
+      case 6: SET_OPTION(CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V6); break;
+    }
+  }
+
  /**
   * Connect timeout
   */
@@ -928,16 +975,16 @@ Url MediaCurl::getFileUrl( const Pathname & filename_r ) const
 
 ///////////////////////////////////////////////////////////////////
 
-void MediaCurl::getFile( const Pathname & filename ) const
+void MediaCurl::getFile(const Pathname & filename , const ByteCount &expectedFileSize_r) const
 {
     // Use absolute file name to prevent access of files outside of the
     // hierarchy below the attach point.
-    getFileCopy(filename, localPath(filename).absolutename());
+    getFileCopy(filename, localPath(filename).absolutename(), expectedFileSize_r);
 }
 
 ///////////////////////////////////////////////////////////////////
 
-void MediaCurl::getFileCopy( const Pathname & filename , const Pathname & target) const
+void MediaCurl::getFileCopy( const Pathname & filename , const Pathname & target, const ByteCount &expectedFileSize_r ) const
 {
   callback::SendReport<DownloadProgressReport> report;
 
@@ -949,7 +996,7 @@ void MediaCurl::getFileCopy( const Pathname & filename , const Pathname & target
   {
     try
     {
-      doGetFileCopy(filename, target, report);
+      doGetFileCopy(filename, target, report, expectedFileSize_r);
       retry = false;
     }
     // retry with proper authentication data
@@ -966,8 +1013,13 @@ void MediaCurl::getFileCopy( const Pathname & filename , const Pathname & target
     // unexpected exception
     catch (MediaException & excpt_r)
     {
-      // FIXME: error number fix
-      report->finish(fileurl, zypp::media::DownloadProgressReport::ERROR, excpt_r.asUserHistory());
+      media::DownloadProgressReport::Error reason = media::DownloadProgressReport::ERROR;
+      if( typeid(excpt_r) == typeid( media::MediaFileNotFoundException )  ||
+	  typeid(excpt_r) == typeid( media::MediaNotAFileException ) )
+      {
+	reason = media::DownloadProgressReport::NOT_FOUND;
+      }
+      report->finish(fileurl, reason, excpt_r.asUserHistory());
       ZYPP_RETHROW(excpt_r);
     }
   }
@@ -1009,9 +1061,9 @@ bool MediaCurl::getDoesFileExist( const Pathname & filename ) const
 
 ///////////////////////////////////////////////////////////////////
 
-void MediaCurl::evaluateCurlCode( const Pathname &filename,
+void MediaCurl::evaluateCurlCode(const Pathname &filename,
                                   CURLcode code,
-                                  bool timeout_reached ) const
+                                  bool timeout_reached) const
 {
   if ( code != 0 )
   {
@@ -1020,8 +1072,8 @@ void MediaCurl::evaluateCurlCode( const Pathname &filename,
       url = _url;
     else
       url = getFileUrl(filename);
+
     std::string err;
-    try
     {
       switch ( code )
       {
@@ -1057,6 +1109,7 @@ void MediaCurl::evaluateCurlCode( const Pathname &filename,
                            ));
           }
 
+          case 502: // bad gateway (bnc #1070851)
           case 503: // service temporarily unavailable (bnc #462545)
             ZYPP_THROW(MediaTemporaryProblemException(url));
           case 504: // gateway timeout
@@ -1069,6 +1122,7 @@ void MediaCurl::evaluateCurlCode( const Pathname &filename,
             ZYPP_THROW(MediaForbiddenException(url, msg403));
           }
           case 404:
+          case 410:
               ZYPP_THROW(MediaFileNotFoundException(_url, filename));
           }
 
@@ -1129,10 +1183,6 @@ void MediaCurl::evaluateCurlCode( const Pathname &filename,
       // uhm, no 0 code but unknown curl exception
       ZYPP_THROW(MediaCurlException(url, err, _curlError));
     }
-    catch (const MediaException & excpt_r)
-    {
-      ZYPP_RETHROW(excpt_r);
-    }
   }
   else
   {
@@ -1332,7 +1382,7 @@ bool MediaCurl::detectDirIndex() const
 
 ///////////////////////////////////////////////////////////////////
 
-void MediaCurl::doGetFileCopy( const Pathname & filename , const Pathname & target, callback::SendReport<DownloadProgressReport> & report, RequestOptions options ) const
+void MediaCurl::doGetFileCopy(const Pathname & filename , const Pathname & target, callback::SendReport<DownloadProgressReport> & report, const ByteCount &expectedFileSize_r, RequestOptions options ) const
 {
     Pathname dest = target.absolutename();
     if( assert_dir( dest.dirname() ) )
@@ -1384,7 +1434,7 @@ void MediaCurl::doGetFileCopy( const Pathname & filename , const Pathname & targ
     }
     try
     {
-      doGetFileCopyFile(filename, dest, file, report, options);
+      doGetFileCopyFile(filename, dest, file, report, expectedFileSize_r, options);
     }
     catch (Exception &e)
     {
@@ -1446,7 +1496,7 @@ void MediaCurl::doGetFileCopy( const Pathname & filename , const Pathname & targ
 
 ///////////////////////////////////////////////////////////////////
 
-void MediaCurl::doGetFileCopyFile( const Pathname & filename , const Pathname & dest, FILE *file, callback::SendReport<DownloadProgressReport> & report, RequestOptions options ) const
+void MediaCurl::doGetFileCopyFile(const Pathname & filename , const Pathname & dest, FILE *file, callback::SendReport<DownloadProgressReport> & report, const ByteCount &expectedFileSize_r, RequestOptions options ) const
 {
     DBG << filename.asString() << endl;
 
@@ -1486,7 +1536,7 @@ void MediaCurl::doGetFileCopyFile( const Pathname & filename , const Pathname &
     }
 
     // Set callback and perform.
-    ProgressData progressData(_curl, _settings.timeout(), url, &report);
+    ProgressData progressData(_curl, _settings.timeout(), url, expectedFileSize_r, &report);
     if (!(options & OPTION_NO_REPORT_START))
       report->start(url, dest);
     if ( curl_easy_setopt( _curl, CURLOPT_PROGRESSDATA, &progressData ) != 0 ) {
@@ -1530,7 +1580,11 @@ void MediaCurl::doGetFileCopyFile( const Pathname & filename , const Pathname &
       // which holds whether the timeout was reached or not,
       // otherwise it would be a user cancel
       try {
-        evaluateCurlCode( filename, ret, progressData.reached);
+
+        if ( progressData.fileSizeExceeded )
+          ZYPP_THROW(MediaFileSizeExceededException(url, progressData._expectedFileSize));
+
+        evaluateCurlCode( filename, ret, progressData.reached );
       }
       catch ( const MediaException &e ) {
         // some error, we are not sure about file existence, rethrw
@@ -1560,7 +1614,7 @@ void MediaCurl::getDir( const Pathname & dirname, bool recurse_r ) const
       switch ( it->type ) {
       case filesystem::FT_NOT_AVAIL: // old directory.yast contains no typeinfo at all
       case filesystem::FT_FILE:
-        getFile( filename );
+        getFile( filename, 0 );
         break;
       case filesystem::FT_DIR: // newer directory.yast contain at least directory info
         if ( recurse_r ) {
@@ -1650,13 +1704,24 @@ string MediaCurl::getAuthHint() const
   return "";
 }
 
+/**
+ * MediaMultiCurl needs to reset the expected filesize in case a metalink file is downloaded
+ * otherwise this function should not be called
+ */
+void MediaCurl::resetExpectedFileSize(void *clientp, const ByteCount &expectedFileSize)
+{
+  ProgressData *data = reinterpret_cast<ProgressData *>(clientp);
+  if ( data ) {
+    data->_expectedFileSize = expectedFileSize;
+  }
+}
+
 ///////////////////////////////////////////////////////////////////
 
 bool MediaCurl::authenticate(const string & availAuthTypes, bool firstTry) const
 {
   //! \todo need a way to pass different CredManagerOptions here
-  Target_Ptr target = zypp::getZYpp()->getTarget();
-  CredentialManager cm(CredManagerOptions(target ? target->root() : ""));
+  CredentialManager cm(CredManagerOptions(ZConfig::instance().repoManagerRoot()));
   CurlAuthData_Ptr credentials;
 
   // get stored credentials
