@@ -55,6 +55,9 @@ namespace zypp
     const SerialNumber & Pool::serial() const
     { return myPool().serial(); }
 
+    const SerialNumber & Pool::serialIDs() const
+    { return myPool().serialIDs(); }
+
     void Pool::prepare() const
     { return myPool().prepare(); }
 
@@ -250,7 +253,7 @@ namespace zypp
       return str << "sat::pool(" << obj.serial() << ")["
           << obj.capacity() << "]{"
           << obj.reposSize() << "repos|"
-	  << obj.solvablesSize() << "slov}";
+	  << obj.solvablesSize() << "solv}";
     }
 
     /////////////////////////////////////////////////////////////////
