@@ -66,12 +66,10 @@ std::ostream & Resolver::dumpOn( std::ostream & os ) const
 {
   os << "<resolver>" << endl;
   #define OUTS(t) os << "  " << #t << ":\t" << t << endl;
-  OUTS( _forceResolve );
   OUTS( _upgradeMode );
   OUTS( _updateMode );
   OUTS( _verifying );
   OUTS( _onlyRequires );
-  OUTS( _allowVendorChange );
   OUTS( _solveSrcPackages );
   OUTS( _cleandepsOnRemove );
   OUTS( _ignoreAlreadyRecommended );
@@ -86,12 +84,10 @@ Resolver::Resolver (const ResPool & pool)
     : _pool(pool)
     , _satResolver(NULL)
     , _poolchanged(_pool.serial() )
-    , _forceResolve		(false)
     , _upgradeMode		(false)
     , _updateMode		(false)
     , _verifying		(false)
     , _onlyRequires		( ZConfig::instance().solver_onlyRequires() )
-    , _allowVendorChange	( ZConfig::instance().solver_allowVendorChange() )
     , _solveSrcPackages		( false )
     , _cleandepsOnRemove	( ZConfig::instance().solver_cleandepsOnRemove() )
     , _ignoreAlreadyRecommended	( true )
@@ -115,6 +111,14 @@ Resolver::~Resolver()
     bool Resolver::ZGETTER() const							\
     { return _satResolver->ZVARNAME; }							\
 
+// NOTE: ZVARDEFAULT must be in sync with SATResolver ctor
+ZOLV_FLAG_TRIBOOL( setForceResolve,		forceResolve,		_allowuninstall,	false )
+
+ZOLV_FLAG_TRIBOOL( setAllowDowngrade,		allowDowngrade,		_allowdowngrade,	false )
+ZOLV_FLAG_TRIBOOL( setAllowNameChange,		allowNameChange,	_allownamechange,	true )	// bsc#1071466
+ZOLV_FLAG_TRIBOOL( setAllowArchChange,		allowArchChange,	_allowarchchange,	false )
+ZOLV_FLAG_TRIBOOL( setAllowVendorChange,	allowVendorChange,	_allowvendorchange,	ZConfig::instance().solver_allowVendorChange() )
+
 ZOLV_FLAG_TRIBOOL( dupSetAllowDowngrade,	dupAllowDowngrade,	_dup_allowdowngrade,	ZConfig::instance().solver_dupAllowDowngrade() )
 ZOLV_FLAG_TRIBOOL( dupSetAllowNameChange,	dupAllowNameChange,	_dup_allownamechange,	ZConfig::instance().solver_dupAllowNameChange() )
 ZOLV_FLAG_TRIBOOL( dupSetAllowArchChange,	dupAllowArchChange,	_dup_allowarchchange,	ZConfig::instance().solver_dupAllowArchChange() )
@@ -123,11 +127,6 @@ ZOLV_FLAG_TRIBOOL( dupSetAllowVendorChange,	dupAllowVendorChange,	_dup_allowvend
 #undef ZOLV_FLAG_TRIBOOL
 //---------------------------------------------------------------------------
 
-void Resolver::setAllowVendorChange( TriBool state_r )
-{
-  _allowVendorChange = indeterminate(state_r) ? ZConfig::instance().solver_allowVendorChange() : bool(state_r);
-}
-
 void Resolver::setOnlyRequires( TriBool state_r )
 {
   _onlyRequires = indeterminate(state_r) ? ZConfig::instance().solver_onlyRequires() : bool(state_r);
@@ -308,13 +307,7 @@ void Resolver::solverInit()
     _satResolver->setFixsystem			( isVerifyingMode() );
     _satResolver->setIgnorealreadyrecommended	( ignoreAlreadyRecommended() );
     _satResolver->setOnlyRequires		( onlyRequires() );
-    _satResolver->setAllowdowngrade		(false);
-    _satResolver->setAllowarchchange		(false);
-    _satResolver->setAllowvendorchange		( allowVendorChange() );
-    _satResolver->setAllowuninstall		( forceResolve() );
-    _satResolver->setUpdatesystem		(false);
-    _satResolver->setNoupdateprovide		(false);
-    _satResolver->setDosplitprovides		(true);
+    _satResolver->setUpdatesystem		(_updateMode);
     _satResolver->setSolveSrcPackages		( solveSrcPackages() );
     _satResolver->setCleandepsOnRemove		( cleandepsOnRemove() );
 
