@@ -143,13 +143,13 @@ SATResolver::dumpOn( std::ostream & os ) const
 	OUTS( ALLOW_DOWNGRADE );
 	OUTS( ALLOW_ARCHCHANGE );
 	OUTS( ALLOW_VENDORCHANGE );
+	OUTS( ALLOW_NAMECHANGE );
 	OUTS( ALLOW_UNINSTALL );
 	OUTS( NO_UPDATEPROVIDE );
 	OUTS( SPLITPROVIDES );
 	OUTS( IGNORE_RECOMMENDED );
 	OUTS( ADD_ALREADY_RECOMMENDED );
 	OUTS( NO_INFARCHCHECK );
-	OUTS( ALLOW_NAMECHANGE );
 	OUTS( KEEP_EXPLICIT_OBSOLETES );
 	OUTS( BEST_OBEY_POLICY );
 	OUTS( NO_AUTOTARGET );
@@ -175,18 +175,20 @@ SATResolver::dumpOn( std::ostream & os ) const
 
 //---------------------------------------------------------------------------
 
+// NOTE: flag defaults must be in sync with ZVARDEFAULT in Resolver.cc
 SATResolver::SATResolver (const ResPool & pool, sat::detail::CPool *satPool)
     : _pool(pool)
     , _satPool(satPool)
     , _satSolver(NULL)
     , _fixsystem(false)
-    , _allowdowngrade(false)
-    , _allowarchchange(false)
-    , _allowvendorchange(ZConfig::instance().solver_allowVendorChange())
-    , _allowuninstall(false)
+    , _allowdowngrade		( false )
+    , _allownamechange		( true )	// bsc#1071466
+    , _allowarchchange		( false )
+    , _allowvendorchange	( ZConfig::instance().solver_allowVendorChange() )
+    , _allowuninstall		( false )
     , _updatesystem(false)
-    , _noupdateprovide(false)
-    , _dosplitprovides(true)
+    , _noupdateprovide		( false )
+    , _dosplitprovides		( true )
     , _onlyRequires(ZConfig::instance().solver_onlyRequires())
     , _ignorealreadyrecommended(true)
     , _distupgrade(false)
@@ -393,12 +395,13 @@ SATResolver::solving(const CapabilitySet & requires_caps,
 	queue_push( &(_jobQueue), 0 );
     }
     solver_set_flag(_satSolver, SOLVER_FLAG_ADD_ALREADY_RECOMMENDED, !_ignorealreadyrecommended);
-    solver_set_flag(_satSolver, SOLVER_FLAG_ALLOW_DOWNGRADE, _allowdowngrade);
-    solver_set_flag(_satSolver, SOLVER_FLAG_ALLOW_UNINSTALL, _allowuninstall);
-    solver_set_flag(_satSolver, SOLVER_FLAG_ALLOW_ARCHCHANGE, _allowarchchange);
-    solver_set_flag(_satSolver, SOLVER_FLAG_ALLOW_VENDORCHANGE, _allowvendorchange);
-    solver_set_flag(_satSolver, SOLVER_FLAG_SPLITPROVIDES, _dosplitprovides);
-    solver_set_flag(_satSolver, SOLVER_FLAG_NO_UPDATEPROVIDE, _noupdateprovide);
+    solver_set_flag(_satSolver, SOLVER_FLAG_ALLOW_DOWNGRADE,		_allowdowngrade);
+    solver_set_flag(_satSolver, SOLVER_FLAG_ALLOW_NAMECHANGE,		_allownamechange);
+    solver_set_flag(_satSolver, SOLVER_FLAG_ALLOW_ARCHCHANGE,		_allowarchchange);
+    solver_set_flag(_satSolver, SOLVER_FLAG_ALLOW_VENDORCHANGE,		_allowvendorchange);
+    solver_set_flag(_satSolver, SOLVER_FLAG_ALLOW_UNINSTALL,		_allowuninstall);
+    solver_set_flag(_satSolver, SOLVER_FLAG_NO_UPDATEPROVIDE,		_noupdateprovide);
+    solver_set_flag(_satSolver, SOLVER_FLAG_SPLITPROVIDES,		_dosplitprovides);
     solver_set_flag(_satSolver, SOLVER_FLAG_IGNORE_RECOMMENDED, _onlyRequires);
     solver_set_flag(_satSolver, SOLVER_FLAG_DUP_ALLOW_DOWNGRADE,	_dup_allowdowngrade );
     solver_set_flag(_satSolver, SOLVER_FLAG_DUP_ALLOW_NAMECHANGE,	_dup_allownamechange );
@@ -830,12 +833,13 @@ void SATResolver::doUpdate()
 	queue_push( &(_jobQueue), 0 );
     }
     solver_set_flag(_satSolver, SOLVER_FLAG_ADD_ALREADY_RECOMMENDED, !_ignorealreadyrecommended);
-    solver_set_flag(_satSolver, SOLVER_FLAG_ALLOW_DOWNGRADE, _allowdowngrade);
-    solver_set_flag(_satSolver, SOLVER_FLAG_ALLOW_UNINSTALL, _allowuninstall);
-    solver_set_flag(_satSolver, SOLVER_FLAG_ALLOW_ARCHCHANGE, _allowarchchange);
-    solver_set_flag(_satSolver, SOLVER_FLAG_ALLOW_VENDORCHANGE, _allowvendorchange);
-    solver_set_flag(_satSolver, SOLVER_FLAG_SPLITPROVIDES, _dosplitprovides);
-    solver_set_flag(_satSolver, SOLVER_FLAG_NO_UPDATEPROVIDE, _noupdateprovide);
+    solver_set_flag(_satSolver, SOLVER_FLAG_ALLOW_DOWNGRADE,		_allowdowngrade);
+    solver_set_flag(_satSolver, SOLVER_FLAG_ALLOW_NAMECHANGE,		_allownamechange);
+    solver_set_flag(_satSolver, SOLVER_FLAG_ALLOW_ARCHCHANGE,		_allowarchchange);
+    solver_set_flag(_satSolver, SOLVER_FLAG_ALLOW_VENDORCHANGE,		_allowvendorchange);
+    solver_set_flag(_satSolver, SOLVER_FLAG_ALLOW_UNINSTALL,		_allowuninstall);
+    solver_set_flag(_satSolver, SOLVER_FLAG_NO_UPDATEPROVIDE,		_noupdateprovide);
+    solver_set_flag(_satSolver, SOLVER_FLAG_SPLITPROVIDES,		_dosplitprovides);
     solver_set_flag(_satSolver, SOLVER_FLAG_IGNORE_RECOMMENDED, _onlyRequires);
 
     sat::Pool::instance().prepare();
@@ -1077,11 +1081,9 @@ string SATResolver::SATprobleminfoString(Id problem, string &detail, Id &ignoreI
 	  }
 	  if (providerlistUninstalled.size() > 0) {
 	      if (detail.size() > 0)
-		  // translators: 'uninstallable' == 'not installable'
-		  detail += _("\nuninstallable providers: ");
+		  detail += _("\nnot installable providers: ");
 	      else
-		  // translators: 'uninstallable' == 'not installable'
-		  detail = _("uninstallable providers: ");
+		  detail = _("not installable providers: ");
 	      for (ProviderList::const_iterator iter = providerlistUninstalled.begin(); iter != providerlistUninstalled.end(); iter++) {
 		  if (iter == providerlistUninstalled.begin())
 		      detail += itemToString( *iter );
