@@ -38,6 +38,7 @@
 #include "zypp/RepoStatus.h"
 #include "zypp/ExternalProgram.h"
 #include "zypp/Repository.h"
+#include "zypp/ShutdownLock_p.h"
 
 #include "zypp/ResFilters.h"
 #include "zypp/HistoryLog.h"
@@ -914,7 +915,7 @@ namespace zypp
           cmd.push_back( _root.asString() );
         }
         cmd.push_back( "-X" );	// autogenerate pattern/product/... from -package
-        cmd.push_back( "-A" );	// autogenerate application pseudo packages
+        // bsc#1104415: no more application support // cmd.push_back( "-A" );	// autogenerate application pseudo packages
         cmd.push_back( "-p" );
         cmd.push_back( Pathname::assertprefix( _root, "/etc/products.d" ).asString() );
 
@@ -1095,6 +1096,7 @@ namespace zypp
     {
       // ----------------------------------------------------------------- //
       ZYppCommitPolicy policy_r( policy_rX );
+      ShutdownLock lck("Zypp commit running.");
 
       // Fake outstanding YCP fix: Honour restriction to media 1
       // at installation, but install all remaining packages if post-boot.
@@ -1234,7 +1236,7 @@ namespace zypp
       if ( ! policy_r.dryRun() || policy_r.downloadMode() == DownloadOnly )
       {
 	// Prepare the package cache. Pass all items requiring download.
-        CommitPackageCache packageCache( root() );
+        CommitPackageCache packageCache;
 	packageCache.setCommitList( steps.begin(), steps.end() );
 
         bool miss = false;
@@ -1264,22 +1266,7 @@ namespace zypp
               ManagedFile localfile;
               try
               {
-		// TODO: unify packageCache.get for Package and SrcPackage
-		if ( pi->isKind<Package>() )
-		{
-		  localfile = packageCache.get( pi );
-		}
-		else if ( pi->isKind<SrcPackage>() )
-		{
-		  repo::RepoMediaAccess access;
-		  repo::SrcPackageProvider prov( access );
-		  localfile = prov.provideSrcPackage( pi->asKind<SrcPackage>() );
-		}
-		else
-		{
-		  INT << "Don't know howto cache: Neither Package nor SrcPackage: " << pi << endl;
-		  continue;
-		}
+		localfile = packageCache.get( pi );
                 localfile.resetDispose(); // keep the package file in the cache
               }
               catch ( const AbortRequestException & exp )
@@ -1614,10 +1601,9 @@ namespace zypp
 
       } // for
 
-      // process all remembered posttrans scripts.
-      if ( !abort )
-	postTransCollector.executeScripts();
-      else
+      // process all remembered posttrans scripts. If aborting,
+      // at least log omitted scripts.
+      if ( abort || (abort = !postTransCollector.executeScripts()) )
 	postTransCollector.discardScripts();
 
       // Check presence of update scripts/messages. If aborting,
@@ -1798,7 +1784,7 @@ namespace zypp
       {
         // ...But the baseproduct method is not expected to work on RedHat derivatives.
         // On RHEL, Fedora and others the "product version" is determined by the first package
-        // providing 'redhat-release'. This value is not hardcoded in YUM and can be configured
+        // providing 'system-release'. This value is not hardcoded in YUM and can be configured
         // with the $distroverpkg variable.
         scoped_ptr<rpm::RpmDb> tmprpmdb;
         if ( ZConfig::instance().systemRoot() == Pathname() )
