@@ -82,6 +82,9 @@ namespace zypp
     PoolItem Selectable::highestAvailableVersionObj() const
     { return _pimpl->highestAvailableVersionObj(); }
 
+    bool Selectable::identIsAutoInstalled() const
+    { return _pimpl->identIsAutoInstalled(); }
+
     bool Selectable::identicalAvailable( const PoolItem & rhs ) const
     { return _pimpl->identicalAvailable( rhs ); }
 
@@ -152,6 +155,21 @@ namespace zypp
     Selectable::picklist_iterator Selectable::picklistEnd() const
     { return _pimpl->picklistEnd(); }
 
+    Selectable::picklist_size_type Selectable::picklistPos( const PoolItem & pi_r ) const
+    { return picklistPos( pi_r.satSolvable() ); }
+
+    Selectable::picklist_size_type Selectable::picklistPos( const sat::Solvable & solv_r ) const
+    {
+      picklist_size_type idx = picklist_size_type(0);
+      for ( const auto & pi : picklist() )
+      {
+	if ( pi == solv_r )
+	  return idx;
+	++idx;
+      }
+      return picklistNoPos;
+    }
+
     ////////////////////////////////////////////////////////////////////////
 
     bool Selectable::isUnmaintained() const
