@@ -37,7 +37,7 @@
 
 #define RX_VALID_PORT      "^[0-9]{1,5}$"
 
-#define RX_VALID_HOSTNAME  "^[[:alnum:]]+([\\.-][[:alnum:]]+)*$"
+#define RX_VALID_HOSTNAME  "^[[:alnum:]${_}]+([\\.-][[:alnum:]${_}]+)*$"
 
 #define RX_VALID_HOSTIPV4  \
         "^([0-9]{1,3})\\.([0-9]{1,3})\\.([0-9]{1,3})\\.([0-9]{1,3})$"
@@ -256,7 +256,7 @@ namespace zypp
 
       config("safe_username",   "~!$&'()*+=,;");
       config("safe_password",   "~!$&'()*+=,:;");
-      config("safe_hostname",   "[:]");
+      config("safe_hostname",   "[:]${_}");
       config("safe_pathname",   "~!$&'()*+=,:@/");
       config("safe_pathparams", "~!$&'()*+=,:;@/");
       config("safe_querystr",   "~!$&'()*+=,:;@/?");
